;;-----------------------------LICENSE NOTICE------------------------------------
;;  This file is part of Crimson Knight Adventures: An Amstrad CPC Game
;;  Copyright (C) 2018 Carlos Soler Mujeriego / Enrique Vidal Cayuela / Alejandro
;;  Gutiérrez Martínez
;;
;;  This program is free software: you can redistribute it and/or modify
;;  it under the terms of the GNU Lesser General Public License as published by
;;  the Free Software Foundation, either version 3 of the License, or
;;  (at your option) any later version.
;;
;;  This program is distributed in the hope that it will be useful,
;;  but WITHOUT ANY WARRANTY; without even the implied warranty of
;;  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
;;  GNU Lesser General Public License for more details.
;;
;;  You should have received a copy of the GNU Lesser General Public License
;;  along with this program.  If not, see <http://www.gnu.org/licenses/>.
;;-------------------------------------------------------------------------------
.include "main.h.s"
.include "enemy.h.s"
.include "renderer.h.s"
.include "collision.h.s"
.include "player.h.s"
.include "random.h.s"
.include "hud.h.s"

.area _CODE

;; ENEMY DEFAULT (LEFT)
DefineEnemyDefault enemy, _default

;; ENEMY RIGHT      _
DefineEnemy enemy_right, #72, #96, #4, #32, _sprites1_sp_08, #-1, #0, enemy_move, #0, #10, enemy_next_to_player, enemy_delete, #0, #15, #0, #0

;; PROYECTILES
DefineEnemy proyectil_LU,   #4,   #96, #4, #4, _sprites2_sp_1,  #2, #0, enemy_move, #0, #0, proyectil_delete, proyectil_delete, #-1, #15, #0, #1
DefineEnemy proyectil_RU,  #72,   #96, #4, #4, _sprites2_sp_0, #-2, #0, enemy_move, #0, #0, proyectil_delete, proyectil_delete, #-1, #15, #0, #1
DefineEnemy proyectil_LD,   #4,  #120, #4, #4, _sprites2_sp_1,  #2, #0, enemy_move, #0, #0, proyectil_delete, proyectil_delete, #-1, #15, #0, #1
DefineEnemy proyectil_RD,  #72,  #120, #4, #4, _sprites2_sp_0, #-2, #0, enemy_move, #0, #0, proyectil_delete, proyectil_delete, #-1, #15, #0, #1

;; CREATE A VECTOR OF ENEMIES
k_max_num_vect = 3            ;; Maximo numero de enemigos
k_entity_size = 20            ;; Tamaño de las entidades
enemy_vector:                 ;; Vector de enemigos
DefineEnemyVector enemy_vector, k_max_num_vect
num_ent: .db 00               ;; Numero de entidades generadas hasta el momento
next_entity: .dw enemy_vector ;; Puntero a la siguiente entidad

;; VARIABLES DEL GENERADOR
cont: .db #0                  ;; Contador que lleva la cuenta del tiempo que pasa
cont_time: .db #30            ;; Tiempo que ha de pasar para generar un enemigo
cont_left: .db #0             ;; Enemigos que hay a la izquierda

;; BOOSTERS
proyectil_speed: .db  #0
enemy_speed: .db #0

restet_boosters:
   ;; Booster de velocidad de los proyectiles
   ld a, #0
   ld (proyectil_speed), a

   ld a, #0
   ld (enemy_speed), a

ret

enemy_init_level:

   ;;
   ;; RESET BASICO
   ;;
   call enemy_vector_destroy     ;; Destruir vector de enemigos
   ;; Ponemos el contador a 0
   ld a, #0
   ld (cont), a

   ld a, #0
   ld (cont_left), a

   ;; Se ponen los boosters a 0
   call restet_boosters

   ld a, (actual_level)
   cp #1
   jp z, init_level1
   cp #2
   jp z, init_level2
   cp #3
   jp z, init_level3
   cp #4
   jp z, init_level4
   cp #5
   jp z, init_level5
   cp #6
   jp z, init_level5
   cp #7
   jp z, init_level7

   jp fin_init

   init_level1:
   ld a, #24
   ld (cont_time), a
   jp fin_init


   init_level2:
   ld a, #20
   ld (cont_time), a
   jp fin_init

   init_level3:
   ld a, #14
   ld (cont_time), a
   jp fin_init
   
   init_level4:
   ld a, #12
   ld (cont_time), a
   jp fin_init

   init_level5:
   ld a, #12
   ld (cont_time), a

   ld a, #1
   ld (enemy_speed), a
   jp fin_init

   init_level7:
   ld a, #12
   ld (cont_time), a

   ld a, #1
   ld (enemy_speed), a
   ld (proyectil_speed), a

   fin_init:
ret

enemy_vector_generator_update:

   ld a, (actual_level)
   cp #1
   jp z, is_level1
   cp #2
   jp z, is_level2
   cp #3
   jp z, is_level3
   cp #4
   jp z, is_level4
   cp #5
   jp z, is_level5
   cp #6
   jp z, is_level6
   cp #7
   jp z, is_level7
   jp fin_gen_update

   is_level1:
   ;; Flechas derecha (arriba y abajo)
   ld c, #0
   call enemy_vector_generator_level_1_3
   jp fin_gen_update

   is_level2:
   ;; Enemigos derecha
   call enemy_vector_generator_level_2
   jp fin_gen_update

   is_level3:
   ;; Enemigos dos lados
   ld c, #1
   call enemy_vector_generator_level_1_3
   jp fin_gen_update

   is_level4:
   ;; Enemigos y flechas
   call enemy_vector_generator_level_4
   jp fin_gen_update

   is_level5:
   ;; Enemigos rapidos ambos lados
   ld c, #1
   call enemy_vector_generator_level_1_3
   jp fin_gen_update
   
   is_level6:
   ;; Enemigos rapidos y flechas
   call enemy_vector_generator_level_4
   jp fin_gen_update

   is_level7:
   ;; Enemigos y flechas rapidos
   call enemy_vector_generator_level_4 

   fin_gen_update:
ret

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; MODIFICA A LOS ENEMIGOS EN FUNCIÓN DE LOS PARÁMETROS
;; REGISTROS DESTRUIDOS: A
;; ENTRADA: IX -> Puntero a entidad
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
enemy_modify:

   ld a, en_type(ix)
   cp #0
   jp z, type_basic

   ld a, en_vx(ix)
   cp #0
   jp m, vel_minus

      ld a, (proyectil_speed)
      add a, en_vx(ix)
      ld en_vx(ix), a
      jp end_modify

   vel_minus:
      ld a, (proyectil_speed)
      ld b, a
      ld a, en_vx(ix)
      sub b
      ld en_vx(ix), a
      jp end_modify
   
   type_basic:

      ld a, en_vx(ix)
      cp #0
      jp m, vel_minus_enemy

      ld a, (enemy_speed)
      add a, en_vx(ix)
      ld en_vx(ix), a
      jp end_modify

      vel_minus_enemy:
      ld a, (enemy_speed)
      ld b, a
      ld a, en_vx(ix)
      sub b
      ld en_vx(ix), a
      jp end_modify

   end_modify:
ret

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;                                         ;;
;;           LEVEL 1 / 3 GENERATOR         ;;
;;                                         ;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
enemy_vector_generator_level_1_3:
   ld a, (cont) ;; Cargar en A el cont
   cp #0        ;; Comparar si es 0
   jp nz, final_level1 ;; Entonces saltar al final

   ;;Poner el contador en cont_time
   ld a, (cont_time)
   ld (cont), a
   ;;---------------------------------

   ld a, c
   cp #0
   jp z, generador_1

   ld hl, #enemy_default
   ld (p_izquierda), hl
   ld hl, #enemy_right
   ld (p_derecha), hl

   jr start_generate

   generador_1:
   ld hl, #proyectil_RD
   ld (p_izquierda), hl
   ld hl, #proyectil_RU
   ld (p_derecha), hl
   
   start_generate:
   call generate_random_number
   ld a, (random_number)
   ld b, #0xBB                ;; B = 0X55
   sub b                      ;; A - B
   jp z, level1_rigth         ;; SI B >= A, ENTONCES ES A <= 0X55, Y EL RESULTADO ES NEGATIVO O CERO
   jp m, level1_rigth     
   
   ;; Hay que hacer una comprobacion, para que los enemigos no se generen mas de dos veces a la izquierda
   ld a, #0
   ld (cont_left), a

   ld hl, #enemy_check_left_enemies
   call ent_doForAll
   
   ld a, (cont_left)
   cp #2
   jp z, level1_rigth

   ;; SE GENERAN PROYECTILES ARRIBA A LA DERECHA
   p_izquierda = . + 1
   ld hl, #proyectil_RD
   call enemy_vector_push
   jp final_level1

   level1_rigth:
   ;; SE GENERAN PROYECTILES ABAJO A LA DERECHA
   p_derecha = . + 1
   ld hl, #proyectil_RU
   call enemy_vector_push

   ;;---------------------------------
   final_level1:
   ld a, (cont)
   dec a
   ld (cont), a

ret

enemy_check_left_enemies:

   ld a, en_vx(ix)
   cp #0
   jp m, end_check

   ld a, (cont_left)
   inc a
   ld (cont_left), a

   end_check:
ret

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;                                         ;;
;;             LEVEL 2 GENERATOR           ;;
;;                                         ;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
enemy_vector_generator_level_2:
   ld a, (cont) ;; Cargar en A el cont
   cp #0        ;; Comparar si es 0
   jp nz, final_level2 ;; Entonces saltar al final

   ;;Poner el contador en cont_time
   ld a, (cont_time)
   ld (cont), a
   ;;---------------------------------

   ld hl, #enemy_right
   call enemy_vector_push

   ;;---------------------------------
   final_level2:
   ld a, (cont)
   dec a
   ld (cont), a
ret

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;                                         ;;
;;             LEVEL 4 GENERATOR           ;;
;;                                         ;;           
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
enemy_vector_generator_level_4:

ld a, (cont) ;; Cargar en A el cont
cp #0        ;; Comparar si es 0
jp nz, final ;; Entonces saltar al final

;;Poner el contador en cont_time
ld a, (cont_time)
ld (cont), a
;;---------------------------------

   call generate_random_number      ;; Generamos un número aleatorio
   ld a, (random_number)            ;; A = RANDOM (0X00 - 0XFF)

   ld b, #0x10                      ;; B = 0X55
   sub b                            ;; A - B
   jp z, intervalo1                 ;; SI B >= A, ENTONCES ES A <= 0X55, Y EL RESULTADO ES NEGATIVO O CERO
   jp m, intervalo1                  
   
   ld a, (random_number)            ;; A = RANDOM NUMBER
   ld b, #0X3F                      ;; B = 0xAA
   sub b                            ;; A - B
   jp z, intervalo2                 ;; SI B >= A, ENTONCES ES A <= 0XAA, Y EL RESULTADO ES NEGATIVO O CERO
   jp m, intervalo2

   ;; SI NO ESTÁ EN NINGUNO DE LOS INTERVALOS ANTERIORES, ENTONCES INTERVALO 3
   ld c, #2                         ;; PUSH PROYECTIL
   call enemy_vector_decide_enemy
   call enemy_vector_push
   jp final

   intervalo1:
   ;; PUSH LEFT
   ;; Hay que hacer una comprobacion, para que los enemigos no se generen mas de dos veces a la izquierda
   ld a, #0
   ld (cont_left), a

   ld hl, #enemy_check_left_enemies
   call ent_doForAll
   
   ld a, (cont_left)
   cp #2
   jp z, intervalo2

   ld c, #0
   call enemy_vector_decide_enemy                         ;; PUSH ENEMIGO IZQ.
   call enemy_vector_push
   jp final

   intervalo2:
   ld c, #1  
   call enemy_vector_decide_enemy                         ;; PUSH ENEMIGO DER.
   call enemy_vector_push
   jp final
;;----------------------
   final:
ld a, (cont)
dec a
ld (cont), a

ret


;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; METODO QUE AÑADE DEVUELVE UN TIPO DE PROYECTIL 
;; REGISTROS DESTRUIDOS:   TODOS
;; ENTRADA:
;; SALIDA: HL --> puntero al proyectil
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
get_new_proyectil:
   ;; PARTIENDO DE QUE ESTE MÉTODO SE LLAMA DEPUES DE 
   ;; GENERAR UN NUMERO ALEATORIO, Y QUE SE COMPRUEBE
   ;; QUE ESTE ESTA EN EL INTERVALO 0XAB - 0XFF  
   ;; DIVIDIMOS EL INTERVALO EN 4
   ;; INTERVALO 1: 0XAB - 0XC0 --> PROYECTIL IU
   ;; INTERVALO 2: 0XC1 - 0XD5 --> PROYECTIL RU
   ;; INTERVALO 3: 0XD6 - 0XEA --> PROYECTIL LD
   ;; INTERVALO 4: 0XEB - 0XFF --> PROYECTIL RD
   ld a, (random_number)
   ld b, #0x6F
   sub b
   jp z, p_intervalo1                    ;; SI 0xC0 >= A, ENTONCES ES A <= 0X55, Y EL RESULTADO ES NEGATIVO O CERO
   jp m, p_intervalo1

   ld a, (random_number)
   ld b, #0x9F
   jp z, p_intervalo2                    
   jp m, p_intervalo2

   ld a, (random_number)
   ld b, #0xCF
   jp z, p_intervalo3                   
   jp m, p_intervalo3

   ld hl, #proyectil_RD
   jp fin_get_p

   p_intervalo1:
   ld hl, #proyectil_LU
   jp fin_get_p
   p_intervalo2:
   ld hl, #proyectil_RU
   jp fin_get_p
   p_intervalo3:
   ld hl, #proyectil_LD
   jp fin_get_p

   fin_get_p:
ret
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; METODO QUE DECIDE QUE ENTIDAD DEVUELVE DEPENDIENDO
;; DEL VALOR DE ENTRADA
;; REGISTROS DESTRUIDOS:   TODOS
;; ENTRADA: C -> 0 (izquierda), 1 (derecha), 2 (proyectil)
;; SALIDA:  HL -> enemyigo de vuelta
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
enemy_vector_decide_enemy:

   ;; Comprueba si c = 0
   ld a, c
   cp #0
   jp z, push_left               ;; Si es 0, push_left
   cp #1                       
   jp z, push_right              ;; Si es 1, push_right   

   ;; Sino, cargamos proyectil
   call get_new_proyectil
   jp do_push

   push_left:
   ld hl, #enemy_default
   jp do_push

   push_right:
   ld hl, #enemy_right
   
   ;; Aquí empieza el push
   do_push:
ret

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;    RUTINA BASICA DE LAS ENTIDADES ENEMIGAS
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
enemy_vector_handler:
   call enemy_vector_update
   call enemy_vector_draw
ret

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;    RUTINA DE LAS COLISIONES
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
enemy_vector_collision:
   ld hl, #enemy_collision_handler
	call ent_doForAll
ret

hit_box_vector_collision:
   ld hl, #hit_box_collision_handler
   call ent_doForAll
ret

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;    DIBUJA TODAS LAS ENTIDADES
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
enemy_vector_draw:
   ld hl, #draw_enemy_sprite
   call ent_doForAll
ret

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;    BORRA DE PANTALLA TODAS LAS ENTIDADES
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

enemy_vector_clear:
   ld hl, #clear_box
   call ent_doForAll
ret

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;    ACTUALIZA TODAS LAS ENTIDADES
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
enemy_vector_update:
   call enemy_vector_collision
   
   call hit_box_vector_collision
   
   ld hl, #enemy_update
   call ent_doForAll

   ld hl, #change_enemy_walking_sprite
   call ent_doForAll

ret


;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; GESTOR DE COLISIONES DE LAS ENTIDADES
;; REGISTROS DESTRUIDOS: A
;; ENTRADA: IX -> Puntero a entidad
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
enemy_collision_handler:
   ld iy, #player
   call collision_x
   ld a, c
   cp #0
   jp z, no_collision

   ld h, en_coll_upd_h(ix)
   ld l, en_coll_upd_l(ix)
   ld (jump_col), hl
   jump_col = . + 1
   call 0x00
   no_collision:
   call screen_colision
ret

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; GESTOR DE COLISIONES DE LAS ENTIDADES CON LA HITBOX
;; REGISTROS DESTRUIDOS:
;; ENTRADA: IX -> Puntero a entidad
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
hit_box_collision_handler:
   ld iy, #hit_hitbox
   call collision_x
   ld a, c
   cp #0
   jp z, no_hitbox_collision
   
   ld h, en_hit_upd_h(ix)
   ld l, en_hit_upd_l(ix)
   ld (jump_hit), hl
   jump_hit = . + 1
   call 0x00
   call inc_decenas
   no_hitbox_collision:
ret

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; PONE AL ENEMIGO AL LADO DEL PLAYER
;; REGISTROS DESTRUIDOS: A
;; ENTRADA: IX -> Puntero a entidad
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
enemy_next_to_player:

   ;;Comprobando si es la primera vez que colisiona
   ld a, collision(ix)
   cp #0
   jr nz, end_collision
   ;;Decimos que ha colisionado
   ld a, #1
   ld collision(ix), a

   ;;Si el enemigo esta a la izquierda, su velocidad es positiva
   ld a, en_vx(ix)
   cp #0
   jp m, es_derecha
   ;;Esto es izquierda, entonces la nueva posicion es Ex = Px - Ew
   ld iy, #player
   ld a, e_x(iy)     ;; A = Px
   ld b, en_w(ix)    ;; B = Ew
   sub b             ;; A - B
   ld en_x(ix), a    ;; Ponemos la Ex del enemigo
   jp continue_collision
   es_derecha:
   ;; Esto es derecha entonces la nueva posicion es Ex = Px + Pw
   ld iy, #player
   ld a, e_x(iy)     ;; A = Px
   ld b, e_w(iy)     ;; B = Pw
   add a, b          ;; A + B
   ld en_x(ix), a    ;; Ponemos la Ex del enemigo

   continue_collision:

   ;;Las siguientes lineas de codigo consideran que el enemigo se ha chocado contra un JUGADOR.
   ;;ld a, (is_player_movement_allowed)
   ;;inc a ;;Negamos el movimiento del personaje mientras nos estemos chocando. *ScreenMovementRegulation*
   ;;ld (is_player_movement_allowed), a ;;*ScreenMovementRegulation*l

   end_collision:
   call enemy_attack

ret


;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; METODO QUE HACE DAÑO AL JUGADOR CADA X IT.
;; REGISTROS DESTRUIDOS: B, A, IY
;; ENTRADA: IX -> Puntero a entidad
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
enemy_attack:
   ld a, hit_lag_cont(ix)
   cp #0
   jp nz, no_attack

   ld a, en_damage(ix)
   ld b, a
   call damage_player
  
   ld a, hit_lag(ix)
   ld hit_lag_cont(ix), a
   jp end_attack

   no_attack:
   dec a
   ld hit_lag_cont(ix), a

   end_attack:
ret

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; DEJA EL VECTOR DE ENEMIGOS VACIO
;; REGISTROS DESTRUIDOS: A, HL
;; ENTRADA:
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
enemy_vector_destroy:
   ;; Ponemos el contador de enemigos del vector a 0
   ld a, #0
   ld (num_ent), a
   ;; Le decimos que next_entity es el principio del vector
   ld hl, #enemy_vector
   ld (next_entity), hl
ret

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; BORRA UNA ENTIDAD
;; REGISTROS DESTRUIDOS:
;; ENTRADA: IX -> Puntero a la entidad que se borra
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
enemy_delete:
   ld a, (num_ent)
   cp #0
   jp z, cancel_delete

   ;;Restar 1 al vector
   dec a
   ld (num_ent), a

   ld (ent_deleted), ix
   ;;call clear_box

   ld a, en_vx(ix)
   cp #0
   jp m, continue_delete

   ld a, (cont_left)
   dec a
   ld (cont_left), a

   continue_delete:
   ;; En este momento, hemos borrado la entidad
   ;; Ahora hay que hacer que el puntero a la última entidad
   ;; cambie a uno menos
   ld hl, (next_entity)
   ld bc, #k_entity_size
   or a
   sbc hl, bc
   ld (next_entity), hl

   ;;Copiando la última entidad en esta
   ld hl, (next_entity)
   ent_deleted = . + 1
   ld de, #0x0000
   call enemy_copy

   ;;ld a, (is_player_movement_allowed)  ;; Permitimos que el jugador se mueva de nuevo. *ScreenMovementRegulation*.
   ;;dec a
   ;;cp #0
   ;;jp m, hacerlo_cero
   ;;ld (is_player_movement_allowed), a
   ;;jr cancel_delete

   ;;hacerlo_cero:
   ;;ld a, #0
   ;;ld (is_player_movement_allowed), a

   cancel_delete:
ret


;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; HACE UNA FUNCION PARA TODAS LAS ENTIDADES DEL VECTOR
;; REGISTROS DESTRUIDOS: AF, HL, DE, BC
;; ENTRADA: HL -> Puntero a funcion
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
ent_doForAll:
   ld  a, (num_ent)
   ;; Primero comprobamos que no este vacio
   cp #0
   jp z, end_doForAll

   ld ix, #enemy_vector          ;; Puntero a la primera entidad del vector
   ld (metodo), hl               ;; Cargamos el metodo en su lugar

   loop_forAll:
      push af                    ;; Push AF a la pila, porque ent_draw destruye AF
      metodo = . + 1
      call draw_box              ;; Dibujamos la primera entidad
      ;;call draw_enemy_sprite

      pop  af
      ld   bc, #k_entity_size    ;; Cargamos el tamaño de una entidad
      add  ix, bc                ;; Sumamos al puntero, para obtener la siguiente entidad
      dec  a
      jp   nz, loop_forAll       ;; Mientras no sea 0, continuamos

   end_doForAll:
ret

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; METODO QUE AÑADE UNA NUEVA ENTIDAD AL VECTOR
;; COMPROBANDO SI HAY ESPACIO O NO
;; REGISTROS DESTRUIDOS:   TODOS
;; ENTRADA: HL -> entidad que se pushea
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
enemy_vector_push:

   ld (enemy_pushed), hl      ;; Carga la entidad de hl en la que se quiere hacer el copy
   
   ld a, (num_ent)            ;; Se comprueba si se puede hacer el push
   cp #k_max_num_vect
   jp z, cancel_push

   ;; Crear una nueva entidad
   call enemy_new
   ex de, hl                  ;; DE = PUNTERO AL PRINCIPIO DE LA ENTIDAD VACÍA DONDE SE VA A COPIAR
   ;; Guardar en ix lo que hay en DE
   ld ix, #0x0000
   add ix, de
   ;;---------------------
   enemy_pushed = . + 1
   ld hl, #enemy_default
   call enemy_copy
   ;;---------------------

   ;; Modificar atributos en funcion de los boost
   call enemy_modify

   cancel_push:
ret

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; COPIA UNA ENTIDAD DESDE HL A DE, EL NUEMRO DE BYTES
;; QUE HAYA EN DE
;; REGISTROS DESTRUIDOS: A, HL, DE
;; ENTRADA: 
;;       HL -> Entidad Origen  (de la que se quiere copiar)
;;       DE -> Entidad Destino (sobre la que se copia)
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
enemy_copy:
   ld bc, #k_entity_size
   ldir
   ret

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; RESERVA ESPACIO PARA UNA NUEVA ENTIDAD EN EL VECTOR
;; REGISTROS DESTRUIDOS: A, HL, DE
;; ENTRADA: IX -> Puntero a entidad
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
enemy_new:
   ;; Aumentamos el numero de entidades
   ld a, (num_ent)
   inc a
   ld (num_ent), a

   ;;Calculamos el puntero a la siguiente entidad
   ld hl, (next_entity)  
   ld bc, #k_entity_size
   add hl, bc                 ;; Sumamos entity_vector con num_ent
   ld (next_entity), hl

   ;;Devolvemos puntero a la entidad actual
   or a                       ;; Deja el acarreo a 0
   sbc hl, bc                 ;; Puntero a la entidad actual (vacía)
 
 ret

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; MOVER UNA ENTIDAD
;; REGISTROS DESTRUIDOS: A,B (*ScreenMovementRegulation*)
;; ENTRADA: IX -> Puntero a entidad
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
enemy_move:
   ;; Comrobar si has colisionado
   ld a, collision(ix)
   cp #1
   jp z, cancel_move

	;;Movement in X
	ld 	a, en_x(ix)
	add   en_vx(ix)
   ld    en_x(ix), a

   cancel_move:
ret

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; MOVER UNA ENTIDAD
;; REGISTROS DESTRUIDOS: HL
;; ENTRADA: IX -> Puntero a entidad
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
enemy_update:
	ld h, en_upd_h(ix)
	ld l, en_upd_l(ix)
jp (hl)

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; METODO QUE SE EJECUTA CUANDO UN PROYECTIL TIENE 
;; IMPACTO CON EL JUGADOR
;; REGISTROS DESTRUIDOS:
;; ENTRADA: IX -> Puntero a entidad
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
proyectil_delete:
   ;; El proyectil hace daño al jugador
   ld a, en_damage(ix)
   ld b, a
   call damage_player
   ;; Se hace un delete del proyectil
   call enemy_delete
ret

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; COMPRUEBA LA COLISION CON LOS BORDES DE LA PANTALLA 
;; REGISTROS DESTRUIDOS:
;; ENTRADA: IX -> Puntero a entidad
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
screen_colision:

   ld a, en_vx(ix)
   cp #0
   jp m, izquierda
   ;; DERECHA
   ;; CONDICIONES
   ;; VX = 1 --> NO HAY COLISION SI PX + PW < 79
   ld a, en_x(ix)       ;; A = PX
   ld b, a              ;; B = PX
   ld a, en_w(ix)       ;; A = PW
   add a, b             ;; PX + PW 
   cp #79               ;; (PX + PW) - 79
   jp m, end_screen
   call enemy_delete
   jp end_screen
   izquierda:
   ;; IZQUIERDA
   ;; CONDICIONES
   ;; VX = -1 --> NO HAY COLISION SI 0 < PX
   ld a, en_x(ix)       ;; A = PX
   ld b, a              ;; B = PX
   ld a, #0             ;; A = 0
   sub b                ;; A - B --> 0 - PX
   jp m, end_screen     ;; SI PX > 0, EL RESULTADO ES NEGATIVO

   call enemy_delete
   end_screen:
ret

change_enemy_walking_sprite:

          ;;Cambiar sprite de movimiento del enemigo
            ld a, en_walking_animation(ix)
            cp #-1
            ret z

            ld a, en_vx(ix)
            cp #0
            jp m, enemy_cambiar_walking_derecha

               ;;El enemigo mira a la izquierda 
               ld a, en_walking_animation(ix)
               cp #2
               jr z, enemy_cambiar_sprite_izquierda_segundo_paso

                  ;;Cambiar sprite de primer paso - derecha

                  ld a, (actual_level)
                  cp #5
                  jp m, primeros_sprite_06_12
                     ld hl, #_sprites1_sp_12
                     jr fin_condicion_sprite_06_12
                     primeros_sprite_06_12:
                     ld hl, #_sprites1_sp_06
                     fin_condicion_sprite_06_12:
                  ld en_sprite_image_h(ix), h
                  ld en_sprite_image_l(ix), l

                  jr enemy_cambiar_walking_fin

                  enemy_cambiar_sprite_izquierda_segundo_paso:
                  ;;Cambiar sprite de segundo paso - derecha

                  ld a, (actual_level)
                  cp #5
                  jp m, primeros_sprite_07_13
                     ld hl, #_sprites1_sp_13
                     jr fin_condicion_sprite_07_13
                     primeros_sprite_07_13:
                     ld hl, #_sprites1_sp_07
                     fin_condicion_sprite_07_13:
                  ld en_sprite_image_h(ix), h
                  ld en_sprite_image_l(ix), l

                  jr enemy_cambiar_walking_fin


               enemy_cambiar_walking_derecha:
               ;;El enemigo mira a la derecha 
               ld a, en_walking_animation(ix)
               cp #2
               jr z, enemy_cambiar_sprite_segundo_paso

                  ;;Cambiar sprite de primer paso - derecha

                  ld a, (actual_level)
                  cp #5
                  jp m, primeros_sprite_08_14
                     ld hl, #_sprites1_sp_14
                     jr fin_condicion_sprite_08_14
                     primeros_sprite_08_14:
                     ld hl, #_sprites1_sp_08
                     fin_condicion_sprite_08_14:                  
                  ld en_sprite_image_h(ix), h
                  ld en_sprite_image_l(ix), l

                  jr enemy_cambiar_walking_fin

                  enemy_cambiar_sprite_segundo_paso:
                  ;;Cambiar sprite de segundo paso - derecha

                  ld a, (actual_level)
                  cp #5
                  jp m, primeros_sprite_09_15
                     ld hl, #_sprites1_sp_15
                     jr fin_condicion_sprite_09_15
                     primeros_sprite_09_15:
                     ld hl, #_sprites1_sp_09
                     fin_condicion_sprite_09_15:                  
                  ld en_sprite_image_h(ix), h
                  ld en_sprite_image_l(ix), l

                  jr enemy_cambiar_walking_fin


      enemy_cambiar_walking_fin:

               ld a, en_walking_animation(ix)
               inc a
               ld en_walking_animation(ix), a

               cp #3
               ret nz
               ld a, #0
               ld en_walking_animation(ix), a
               ret
ret



;;====================================================
;;    CALLING THE METHOD TO MOVING ALONG TILEMAP CONTROL TO ALL ENEMIES      -     call enemies_move_along_tilemap
;;--------------------------------------------------
;;
;; DESC.:      - Llama al metodo que mueve a todos los enemigos 8 pixeles cuando el tilemap se ha movido un tile.
;;
;; RECIBE:     - D: Izquierda o derecha 
;;
;; DESTRUYE:   - AF, IX
;;
;;====================================================


enemies_move_along_tilemap:
   
   ;;A sera +8 o -8
   ;;MUY IMPORTANTE: NECESITA D PARA FUNCIONAR

   ld hl, #enemy_move_along_tilemap
   call ent_doForAll

ret

;;====================================================
;;    MOVING ALONG TILEMAP CONTROL      -     call enemy_move_along_tilemap
;;--------------------------------------------------
;;
;; DESC.:      - Mueve a los enemigos 8 pixeles cuando el tilemap se ha movido un tile.
;;
;; RECIBE:     - A: Izquierda o derecha
;;
;; DESTRUYE:   - AF, IX
;;
;;====================================================

enemy_move_along_tilemap:

   ;;D sera +8 o -8

   ;;En D recibiremos +8 o -8, depende a donde se mueva el mapa con nosotros.

   ;;Actualizamos la X del enemigo
   ld a, en_x(ix)
   add a,d
   ld en_x(ix),a

ret

enemy_vector_check_player_allowed:
   ld a, #0
   ld (is_player_movement_allowed), a

   ld hl, #check_allowed
   call ent_doForAll
ret

check_allowed:

   ld a, collision(ix)
   cp #0
   jp z, fin_allow

      ld a, #1
      ld (is_player_movement_allowed), a

   fin_allow:
ret