;;-----------------------------LICENSE NOTICE------------------------------------
;;  This file is part of CPCtelera: An Amstrad CPC Game Engine 
;;  Copyright (C) 2018 ronaldo / Fremos / Cheesetea / ByteRealms (@FranGallegoBR)
;;
;;  This program is free software: you can redistribute it and/or modify
;;  it under the terms of the GNU Lesser General Public License as published by
;;  the Free Software Foundation, either version 3 of the License, or
;;  (at your option) any later version.
;;
;;  This program is distributed in the hope that it will be useful,
;;  but WITHOUT ANY WARRANTY; without even the implied warranty of
;;  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
;;  GNU Lesser General Public License for more details.
;;
;;  You should have received a copy of the GNU Lesser General Public License
;;  along with this program.  If not, see <http://www.gnu.org/licenses/>.
;;-------------------------------------------------------------------------------
;;============================================
;;
;; INCLUDES
;;
;;============================================
.include "cpctelera.h.s"
.include "main.h.s"
.include "player.h.s"
.include "renderer.h.s"
.include "collision.h.s"
.include "enemy.h.s"
.include "screens.h.s"
.include "random.h.s"
.include "hud.h.s"
.include "mapaPruebas_pack.h.s" ;;DECOMPRESS-TILE TESTS
.include "time.h.s"
.include "level1.h.s"
.include "level2.h.s"
.include "level3.h.s"
.include "level4.h.s"
.include "level5.h.s"


.area _DATA
.area _CODE

;;IMPORTANT NOTES==================
;;This code starts when the song data ends. See cfg/build_config.mk
;;Currently, ROM is erased and data is stored inside.
;;//IMPORTANT NOTES================

;;Recordatorio: El CODIGO del programa empieza en 0x4000 tal y como esta definido en build_config.mk. Se puede cambiar.

;;==========             /ESTRUCTURA                ============== ;;DECOMPRESS-TILE TESTS

game_state: .db #00    ;; 00: Title Screen   01: Game Loop   02: Game Over?
music_frec: .db #5   ;;1 every 12 interruptions =~ 1 every 2 frames
keyboard_frec: .db #06 ;;1 every 6 interruptions =~ 1 every 1 frames

music_play: .db #0   ;; 0 = no music, 1 = music



;;========VARIABLES DE CONTROL DE POSICION Y MOVIMIENTO PERSONAJE


player_movement: .db #00         ;; Indica cuanto quiere el jugador moverse. Los enemigos podran tratar de evitarlo.

player_screen_position: .db #00 ;;Indica la posicion del jugador respecto a la pantalla. 0: 
                                 ;;Centro, 10 y -10 son las esquinas donde la camara se mueve (definidas en constante).

is_player_movement_allowed: .db #00 ;;Indica si el jugador puede o no "moverse". 
                                    ;;0:SI 1 0 más = NO


screen_movement: .db #00 ;;Indica cuando la pantalla se "mueve". Le sirve a los tiles y a los enemigos para actualizarse.
                         ;;00 = No hay movimiento. Positivo: Movimiento "Izquierda". Negativo: Mov. "Derecha"
                         ;;Se actualiza en player.
                        ;;Sirve para controlar si se mueve o no la pantalla y los enemigos.

screen_scroll_position: .db #00  ;; Indica an que posicion del scroll del mapa esta el jugador
                                 ;; Si esta a 00 o FF no se puede mover


;;========VARIABLES DE CONTROL DE FINAL DE NIVEL


end_level_position: .db #0x2A      ;;Indica la posicion del final del nivel. Por defecto: Fin Mapa 1


;;Los finales de los niveles en TILED DEBEN tener 10 tiles despues de la puerta final.

end_level_scroll_map_1_2: .db #0x2A  ;;Nada que ver con la descompresion. Numero de PASOS en los que esta el fin del nivel.
end_level_scroll_map_3: .db #0x2B ;;Nada que ver con la descompresion. Numero de PASOS en los que esta el fin del nivel.
end_level_scroll_map_4: .db #0x39  ;;Nada que ver con la descompresion. Numero de PASOS en los que esta el fin del nivel.
end_level_scroll_map_5_6: .db #0x3A  ;;Nada que ver con la descompresion. Numero de PASOS en los que esta el fin del nivel.
end_level_scroll_map_7: .db #0x51  ;;Nada que ver con la descompresion. Numero de PASOS en los que esta el fin del nivel.


;;========VARIABLES DE CONTROL DE NIVEL


actual_level: .db #01            
num_levels = 7


;;========VARIABLES DE CONTROL DE LAS VIDAS


actual_retrys: .db #03



;;============================================
;;
;; MAIN
;;
;; C640
;; c5f0
;;============================================
_main::
  
;;
;;    INIT
;;
   ld sp, #0x8000

   ;;Deshabilitamos el firmware
   call  cpct_disableFirmware_asm
   ld    c, #00
   call cpct_setVideoMode_asm

   ;;Ponemos la paleta de color que va a utilizar el progrmaa.
   ld hl, #_palette   ;;Como es una variable en C, palette se convierte en _palette. 
                        ;;Definida en palette.c creada a partir de image_conversion.mk
   ld de, #16
   call cpct_setPalette_asm

   ld l, #16
   ld h, #0x14
   call cpct_setPALColour_asm

   ;;Descompresion de los mapas y los tilesets
   ;;ld hl, #_mapaPruebas_pack_end ;;definido en mapaPruebas_pack.s ;;DECOMPRESS-TILE TESTS
   ;;ld de, #decompress_buffer_end ;;Define donde acaba el bucle de descompresion y donde empieza el codigo. ;;DECOMPRESS-TILE TESTS
   ;;call cpct_zx7b_decrunch_s_asm ;;DECOMPRESS-TILE TESTS
   ;;call initialize_tilemap

   ;;Initialize the music controller
   ld de, #_song_ingame ;;Symbol of the song. REMEMBER: MODIFIED builg_config.mk TO START THE PROGRAM WHEN SONG ENDS.
   call cpct_akp_musicInit_asm	;;Initialize music player

   ;;Initialize the ISR method (method that will happen when z80 detects an interruption)
   ld hl, #isr
   call cpct_setInterruptHandler_asm ;;isr will be happening at every interruption. It will handle the keyboard, the music...

;;
;; RESTART GAME
;;
reestartGame: 
   
   ld a, #1
   ;;ld a, #7
   ld (actual_level), a ;;Actualizamos el nivel inicial del juego, 1.
   call change_tilemap ;;Change tilemap NECESITA que se le pase en A el nivel a cambiar. En este caso, el primer nivel.
   ;;call restart_tilemap 

   ld a, #3
   ld (actual_retrys), a ;;Numero de reintentos maximos del juego

   ;; CLEAR BUFFERS
   call clear_buffer_8     
   call clear_buffer_C
   call hud_set_all
   call reset_player_enemies_hud_and_tilemap_properties


;;  END RESTART

;;
;; TITLE SCREEN
;;
titleScreenLoop:
   call clear_buffer_8
   call clear_buffer_C
   call title_screen_loop
   ;; DIBUJA EL HUUUUUUD
   
   call draw_HUDBackground

   ;;.db #0x18, #0xFE
;;
;; GAME LOOP
;;
gameloop:

   call move_player

   call draw_tilemap ;;Dibuja el tilemap, comprueba si debemos hacer scroll y si debemos iniciar contadores para desplazar entidades.

   ;;Reseteo del "movimiento" del jugador que influye al mundo y los enemigos:
   ld a, #00
   ld (player_movement), a          ;; Player movement reset

   ;; ENEMY ROUTINE
   call enemy_vector_handler        ;; The enemies also sets is_screen_movement_allowed to 0(not) or 1(allowed).
   call enemy_vector_generator_update
   
   call enemy_vector_check_player_allowed

   ;; PLAYER ROUTINE
   call ent_clear_all
   call update_player              ;; The player also sets screen_movement to 0, -1 or 1.
   call hud_update
   call ent_draw_all



   call draw_cortinilla
   
   call cpct_waitVSYNC_asm          ;; Wait raster
   call switch_buffers


   ;;Comprueba si esta en el final del nivel
   ld ix, #player
   ld a, (end_level_position)
   ld b, a
   ld a, (screen_scroll_position)
   cp b
   jr nz, end_check_end_level_position   
   
      ;; LLegas al final del nivel
      call clear_buffer_8     
      call clear_buffer_C
      call next_level_screen_loop
      ;; Incrementar el nivel
      ld a, (actual_level)
      inc a
      ld (actual_level), a
      ;; Compararlo con el numero de niveles
      cp #num_levels
      jp z, et_change_tilemap
      jp m, et_change_tilemap      

      ;; Si nos pasamos, hemos ganado
      call clear_buffer_8     
      call clear_buffer_C
      call win_screen_loop
      jp reestartGame

         ;; Si estamos en el nivel menor al ultimo:
      et_change_tilemap:
         call change_tilemap
         call clear_buffer_8
         call clear_buffer_C
         call cpct_waitVSYNC_asm

         call reset_player_enemies_hud_and_tilemap_properties
         ld a, #00
         ld (screen_scroll_position), a
         call draw_HUDBackground


         jp gameloop

   end_check_end_level_position:

      ;;Checking life of the player, if 0, we go to game over.
      ld a, e_life(ix)
      cp #0 ;;Si la vida del personaje al restarle uno da negativo, es que hemos perdido.
      ;;jr nz, gameloop_checkLifeDone ;;Flag m: se activa al ser negativo el resultado.
      jp  z, check_over
      jp  p, gameloop_checkLifeDone ;;Flag m: se activa al ser negativo el resultado.
      
      check_over:
      ld a, (actual_retrys)
      cp #0
      jp nz, the_player_still_has_lives_main

      ;;Si no tienes vidas
      ld    ix, #player
      ld    iy, #hit_hitbox
      call update_player_desgirar_sprites ;;Desgira los sprites is es necesario. Recibe en IX al player y en IY a la hitbox


      jp gameOverLoop

      ;;Si tienes vidas

      the_player_still_has_lives_main:

         dec a
         ld (actual_retrys), a 
         ld    ix, #player
         ld    iy, #hit_hitbox
         call update_player_desgirar_sprites ;;Desgira los sprites is es necesario. Recibe en IX al player y en IY a la hitbox

         ld a, (actual_level)
         jp et_change_tilemap ;;recuerda que esto llama tambien a reset_player_enemies_hud_and_tilemap_properties

      
   gameloop_checkLifeDone:
  
jp gameloop



gameOverLoop:
   call clear_buffer_8
   call clear_buffer_C
   call game_over_screen_loop
   jp reestartGame

;;FIN DE ESTADOS DE JUEGO


;;====================================================
;;    RESET EVERYTHING ON THE LEVEL      -     call reset_player_enemies_hud_and_tilemap_properties
;;--------------------------------------------------
;;
;; DESC.:      - Reinicia las propiedades del personaje, el mapa, los enemigos y el tilemap y el hud. Sirve tanto para reinicio de nivel como para reinicio total.
;;
;;    RECIBE:     - Nada.
;;
;;    DESTRUYE:   - [IX, A] + lo que detruye restart_tilemap
;;====================================================

reset_player_enemies_hud_and_tilemap_properties:

   ;; RESET HUD
   call set_timer

   ;; RESET PLAYER
   call  player_reset

   ;; RESET ENEMIES
   call enemy_init_level
   
   ;; RESET SCREEN SCROLL POSITION (cuantas veces se ha movido la pantalla)
   ld    a, #00
   ld    (screen_scroll_position), a

   ld    a, #00
   ld    (ajuste_tilemap), a
   
   ;; RESET SCREEN MOVEMENT (lo que se debe mover la pantalla)
   ld a, #00
   ld (screen_movement), a



   ;; RESET SCREEN MOVEMENT (posicion del jugador relativa al SUELO, no a la pantalla. Misguiding name)
   ld a, #00
   ld (player_screen_position), a


   ;; RESTET TILEMAP
   call restart_tilemap ;;Este metodo reinicia todos los elementos relativos a la posicion relativa del jugador a un mapa y opradores del tilemap.


ret



;;====================================================
;;    ISR      -     call isr
;;--------------------------------------------------
;;
;; DESC.:      - Metodo que se ejecuta cada interrupcion (6 interrupciones en 1 frame aprox.)
;;
;;    RECIBE:     - Nada.
;;
;;    DESTRUYE:   - [HL, AF, BC, DE, IX] (potencialmente, se supone que se deberian borrar,
                                          ;; pero aprentemente consigue recuperarlos)
;;
;;====================================================

isr:  ;; This function will happen at every interruption


   ld a, (music_play)
   cp #0
   jp z, no_music
   ;;Music reproduction begins

            ;;Because musicPlay deletes every register, we save the alternate registers in the stack:
   ex af,af'
   exx      ;;switch all registers for his alternatives
   push af
   push bc 
   push de
   push hl
   push iy


   ld a, (music_frec)
   dec a  ;;We decrease the variable, our objective is to reproduce the music 1 every 12 interruptions (1 every 2 fotograms) (A fotogram = 6 interrupts.)
   ld (music_frec), a
   jr nz, returnisr

      
      call cpct_akp_musicPlay_asm ;;We play the next sound of the song once every 12 interruptions (this is determined by the song Hz)
                                 ;;Destroys AF, AF', BC, BC', DE, DE', HL, HL'
      ld a, #5
      ld (music_frec), a

   ;;Music reproduction ends

   returnisr:

      ;;Keyboard scan begins

      ;;We recover the registers
      pop iy
      pop hl
      pop de
      pop bc
      pop af
      exx
      ex af, af'

      jr is_music_play

      no_music:
      call cpct_akp_stop_asm

      is_music_play:

      ld a, (keyboard_frec)
      dec a ;;We want to reproduce the keyboard scan once very 6 interruptions (once at every frame)
      ld (keyboard_frec), a
      ret nz
   
      call cpct_scanKeyboard_if_asm ;;Desroys AF, BC, DE, HL (potentially)
      ld a, #6
      ld (keyboard_frec), a

      ;;Keyboard scan ends.

ret