;;-----------------------------LICENSE NOTICE------------------------------------
;;  This file is part of Crimson Knight Adventures: An Amstrad CPC Game
;;  Copyright (C) 2018 Carlos Soler Mujeriego / Enrique Vidal Cayuela / Alejandro
;;  Gutiérrez Martínez
;;
;;  This program is free software: you can redistribute it and/or modify
;;  it under the terms of the GNU Lesser General Public License as published by
;;  the Free Software Foundation, either version 3 of the License, or
;;  (at your option) any later version.
;;
;;  This program is distributed in the hope that it will be useful,
;;  but WITHOUT ANY WARRANTY; without even the implied warranty of
;;  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
;;  GNU Lesser General Public License for more details.
;;
;;  You should have received a copy of the GNU Lesser General Public License
;;  along with this program.  If not, see <http://www.gnu.org/licenses/>.
;;-------------------------------------------------------------------------------
;;============================================
;;
;; INCLUDES
;;
;;============================================
.include "main.h.s"
.include "player.h.s"
.include "cpctelera.h.s"
.include "hud.h.s"

.area _CODE


;;Variables animaciones de sprites:
desgirar_sprites: .db #0 ;;1: Se han girado los sprites antes. 0: No se han girado los sprites antes.

walk_animation: .db #0 ;;Indica que sprite toca en la animacion de movimiento. 0-2: Sprite de paso. 3: Sprite de quieto. Dar sensacion de movimiento.
   ;;/fin variables animaciones sprites

sword_animation: .db #0;;0-2: Mostrar rastro. 2 hacia arriba: No mostrar rastro. Se resetea junto al reset de la hitbox.




DefineEntity player, #38, #96, #4, #32, _sprites1_sp_00, #-1, #1, update_player, #20 
                     ;;X  Y    W    H     C  Estado Facing  Update    Vida   Sprite_image   SpriteW SpriteH

                     ;;OJO: SI CAMBIAS ALGUNA DE LAS PROPIEDADES DE ARRIBA, ACTUALIZA EL METODO DE player_reset
                     
DefineHitBox hit_hitbox, #00, #00, #04, #16, _sprites3_sp_15, #00, #00

;; Salta 20 pixeles de alto
jump_table: .db #-6, #-5, #-4, #-2, #-2, #-1, #00, #1, #2, #2, #4, #5, #6
            .db #0x80      ;; Indica el final del salto


;; Indica si se puede mover o no
no_move: .db #00
punching: .db #00


;;====================================================
;;    PLAYER RESET      -     call player_reset
;;--------------------------------------------------
;;
;; DESC.:      - Resetea las propiedades por defecto del jugador
;;
;; RECIBE:     - Nada.
;;
;; DESTRUYE:   - IX, A
;;
;;====================================================
player_reset:
   ld    ix, #player
   ld    a, #100
   ld    e_life(ix), a     ;;We set the player HP to 100.
   ld    a, #38
   ld    e_x(ix),a
   ld    a, #-1
   ld    e_state(ix),a
   ld    a, #96
   ld    e_y(ix),a
   ld    a, #4
   ld    e_w(ix), a
   ld    a, #32
   ld    e_h(ix), a
   ld    a, #1
   ld    e_facing(ix), a
   ld    a, #0
   ld    (no_move), a
   ld    (punching), a
ret



;;====================================================
;;    CHECK START HIT      -     call check_start_hit
;;--------------------------------------------------
;;
;; DESC.:      - Reliza el update de todas las entidades
;;
;; RECIBE:     - IX: Puntero a la entidad
;;
;; DESTRUYE:   - [HL, AF, BC, DE, IX]
;;
;;====================================================
check_start_hit:
   
   ld iy, #hit_hitbox
   ld a, h_cont(iy)
   cp #00
   jr nz, ya_hay_hitbox
      ld h_cont(iy),   #3
      ld h_exists(iy), #1
      ld a, #01
      ld (punching), a

      ;;Reseteamos la animacion de "rafaga" de la espada para las primeras iteraciones.
      ld a, #0
      ld (sword_animation), a

   ya_hay_hitbox:

ret


;;====================================================
;;    HIT CONTROL      -     call hit_control
;;--------------------------------------------------
;;
;; DESC.:      - Reliza el update de todas las entidades
;;
;; RECIBE:     - IX: Puntero a la entidad
;;
;; DESTRUYE:   - [HL, AF, BC, DE, IX]
;;
;;====================================================
hit_control:

   ld iy, #hit_hitbox
   ld  a, h_cont(iy)
   cp  #00
   jp  z, fin_hit_control
      dec a
      ld h_cont(iy), a
      cp  #1
      jp    p, continuar_hit

         ld h_exists(iy), #00
         ld h_x(iy), #01
         ld h_y(iy), #02
         ret
      
      continuar_hit:

      ld a, #16 ;;Cambiamos la altura del sprite de la hitbox a su altura original.
      ld h_h(iy), a


      ld    ix, #player

            ;;Actualizando sprite:
               ld a, e_state(ix)


               cp #-2
               jr z, player_cambiar_sprite_hit_agachado
               cp #-1
               jr z, player_cambiar_sprite_hit_neutral
               
               ;;Si esta saltando:
                  ld a, #4
                  call change_player_sprite
               

                  ld a, #11
                  call change_hitbox_sprite  ;;Tiene en cuenta que en IY esta hitbox
                  ld a, #32 ;;Cambiamos la altura del sprite de la hitbox
                  ld h_h(iy), a

                  jr player_cambiar_sprite_hit_fin

               ;;Si esta agachado:
               player_cambiar_sprite_hit_agachado:
                  ld a, #16 ;;Cambiamos la altura del sprite
                  ld e_h(ix), a

                  ld a, #111
                  call change_player_sprite


                  ld a, #113 
                  call change_hitbox_sprite  ;;Tiene en cuenta que en IY esta hitbox

                  jr player_cambiar_sprite_hit_fin
               ;;Si no esta saltando ni agachado:
               player_cambiar_sprite_hit_neutral:
               ld a, #3 
               call change_player_sprite
               
               ld a, #115
               call change_hitbox_sprite  ;;Tiene en cuenta que en IY esta hitbox

               ;;Fin actualizar el sprite
            player_cambiar_sprite_hit_fin:


      ;; EN FUNCION DEL FACING MODIFICAL LA X
      ld a, e_facing(ix)
      cp #-1
      jr   nz, no_hit_izquierda
      
         ;;Hit de la izquierda
         ld     a, e_x(ix)
         sub    a, e_w(ix)
         ld h_x(iy), a
         ld     a, e_y(ix)
         ld h_y(iy), a

         ret
      no_hit_izquierda:
      cp #1
      jr  nz, no_hit_derecha

         ;;Hit de la derecha
         ld     a, e_x(ix)
         add    a, e_w(ix)
         ld h_x(iy), a
         ld     a, e_y(ix)
         ld h_y(iy), a

         ret
      no_hit_derecha:

      ret
   fin_hit_control:

   ld a, #00
   ld (punching), a


ret


;;====================================================
;;    CHECK START JUMP      -     call check_start_jump
;;--------------------------------------------------
;;
;; DESC.:      - Reliza el update de todas las entidades
;;
;; RECIBE:     - IX: Puntero a la entidad
;;
;; DESTRUYE:   - [HL, AF, BC, DE, IX]
;;
;;====================================================
check_start_jump:
   ld    ix, #player
   ld    a, e_state(ix)
   cp   #-1
   ret  nz

   ld e_state(ix), #00
   ld    ix, #hit_hitbox
   ld h_h(ix), #32


ret

;;====================================================
;;    MOVING CONTROL      -     call jump_control
;;--------------------------------------------------
;;
;; DESC.:      - Mueve al personaje
;;
;; RECIBE:     - 
;;
;; DESTRUYE:   - A, IX
;;
;;====================================================
move_player:
   
   ld a, (is_player_movement_allowed)
   cp #0 
   ret nz

   
   ld ix, #player

   ld a, (player_movement)

   cp #0
   ret z

   jp m, comprobacion_mover_izquierda
   jr mover_jugador

      comprobacion_mover_izquierda: ;;No podemos movernos a la izquierda si estamos en el extremo izquierdo de la pantalla

      ld a, (screen_scroll_position)
      cp #00
      jr nz, mover_jugador

      ld a, (player_screen_position)
      cp #-16 ;;Tope que podemos movernos cuando estamos en el inicio del mapa y no se hace scroll a la izquierda.
      ret z


   mover_jugador:
      ;;Actualizamos la X del jugador
      ld a, (player_movement)
      ld b,a
      ld a, e_x(ix)
      add a,b
      ld e_x(ix),a

      ld a, (player_movement)
      ld b,a

      ;;Actualizamos la posicion relativa del jugador (0: Centro, 10:Borde derecho maximo, despues se gira la camara...)
      ld a, (player_screen_position)
      add a,b
      ld (player_screen_position),a




ret



;;====================================================
;;    MOVING ALONG TILEMAP CONTROL      -     call jump_control
;;--------------------------------------------------
;;
;; DESC.:      - Mueve al personaje 8 pixeles cuando el tilemap se ha movido un tile.
;;
;; RECIBE:     - A: Izquierda o derecha
;;             - D: Numero que debe cambiar player_scroll 
;;
;; DESTRUYE:   - A, IX
;;
;;====================================================
player_move_along_tilemap:

   ld ix, #player

   ;;En A recibiremos +8 o -8, depende a donde se mueva el mapa con nosotros.
   ld b,a

   ;;Actualizamos la X del jugador
   ld a, e_x(ix)
   add a,b
   ld e_x(ix),a

   ;;Actualizamos la posicion relativa del jugador (0: Centro, 10:Borde derecho maximo, despues se gira la camara...)
   ld a, (player_screen_position)
   add a,b
   ld (player_screen_position),a



ret



;;====================================================
;;    JUMP CONTROL      -     call jump_control
;;--------------------------------------------------
;;
;; DESC.:      - 
;;
;; RECIBE:     - IX: Puntero a la entidad
;;
;; DESTRUYE:   - [HL, AF, BC, DE, IX]
;;
;;====================================================
jump_control:
   ld    ix, #player
   ld      a, e_state(ix)     ;;|
   cp    #-1                  ;;+-No hace nada si NO esta saltando (el estado no es 00)
   ret     z                  ;;|
   cp    #-2                  ;;+-No hace nada si NO esta saltando (el estado no es 00)
   ret     z                  ;;|

   ;;SALTO CHACHIPISTACHI
   ld     hl, #jump_table
   ld      c, a
   ld      b, #0
   add    hl, bc

   ld      a, (hl)
   cp  #0x80
   jr      z, end_of_jump

   ld      b, a
   ld      a, e_y(ix)
   add     b
   ld  e_y(ix), a

   ld      a, e_state(ix)  
   inc     a
   ld   e_state(ix), a

      ;;ld a, #32 ;;Cambiamos la altura del sprite
      ;;ld e_sprite_h(ix), a
      ld a, #2
      call change_player_sprite



ret

   end_of_jump:
   ld e_state(ix), #-1
   ld    ix, #hit_hitbox
   ld h_h(ix), #16

ret


;;====================================================
;;    UPDATE PLAYER      -     call update_player
;;--------------------------------------------------
;;
;; DESC.:      - Reliza el update de todas las entidades
;;
;; RECIBE:     - IX: Puntero a la entidad
;;             - IY: Puntero a hitbox 
;;
;; DESTRUYE:   - [HL, AF, BC, DE, IX]
;;
;;====================================================


update_player:

ld    ix, #player
ld    iy, #hit_hitbox

   call update_player_desgirar_sprites ;;Desgira los sprites is es necesario. Recibe en IX al player y en IY a la hitbox

;;UPDATE

      ;;Ponemos, por defecto, el sprite del personaje quieto y su altura normal (del sprite):
      ld a, #32 ;;Cambiamos la altura del sprite
      ld e_h(ix), a
      ld a, #0
      call change_player_sprite
      ;;Fin actualizar sprite

   
   call check_player_input

   call jump_control

   ld b, e_h(ix)
   ld c, e_w(ix)

   call hit_control

;;FIN UPDATE


   ;;Comprobar si tengo que girar el sprite del personaje:
   ld    ix, #player
   ld    iy, #hit_hitbox

   ld a, e_facing(ix)   
   cp #1
   ret z

   girar_sprite:
   ;;Girar sprite del jugador
   ld h, e_sprite_image_h(ix)
   ld l, e_sprite_image_l(ix)
   ld b, e_h(ix)
   ld c, e_w(ix)
   call cpct_hflipSpriteM0_asm   

   ;;Girar sprite de la hitbox
   ld h, h_sprite_image_h(iy)
   ld l, h_sprite_image_l(iy)
   ld b, h_h(iy)
   ld c, h_w(iy)
   call cpct_hflipSpriteM0_asm     

   ld a, #1
   ld (desgirar_sprites), a ;;Indico que en la siguiente iteracion debo desgirar el sprite actual.
ret



update_player_desgirar_sprites:

   ;;Desgiro el sprite anterior si estaba girado:
   ld a, (desgirar_sprites)
   cp #0
   ret z
      ;;Desgirar al jugador
      ld h, e_sprite_image_h(ix)
      ld l, e_sprite_image_l(ix)
      ld  b, e_h(ix)   ;; Alto sprite
      ld  c, e_w(ix)   ;; Ancho sprite
      call cpct_hflipSpriteM0_asm  

      ;;Desgirar la hitbox
      ld h, h_sprite_image_h(iy)
      ld l, h_sprite_image_l(iy)
      ld  b, h_h(iy)   ;; Alto sprite hitbox
      ld  c, h_w(iy)   ;; Ancho sprite hitbox
      call cpct_hflipSpriteM0_asm  

      ld a, #0
      ld (desgirar_sprites), a ;;Indico que en la siguiente iteracion no debo, de momento, desgirar sprites.

ret


;;====================================================
;;    CHECK PLAYER INPUT     -     call check_player_input
;;--------------------------------------------------
;;
;; DESC.:      - Reliza el update de todas las entidades
;;
;; RECIBE:     - IX: Puntero a la entidad
;;
;; DESTRUYE:   - [HL, AF, BC, DE, IX]
;;
;;====================================================
check_player_input:
   
   

   ld  a, e_state(ix)         ;;|
   cp  #-2                    ;;|
   jr  nz, reset_state        ;;+- Si esta agachado se levanta
      ld e_state(ix), #-1     ;;|
      ld  e_y(ix), #96        ;;
      ld  e_h(ix), #32        ;;
      ld a, #00
      ld (no_move), a
   reset_state:               ;;|



   ld    hl, #Key_P
   call  cpct_isKeyPressed_asm
   jr     z, p_no_pressed
      
      call player_punch_key_actions

   p_no_pressed:

   ld    hl, #Joy0_Fire1
   call  cpct_isKeyPressed_asm
   jr     z, joyfire_no_pressed
      
      
      call player_punch_key_actions

   joyfire_no_pressed:

   ld    hl, #Key_S
   call  cpct_isKeyPressed_asm
   jr     z, s_no_pressed

         call player_down_key_actions

   s_no_pressed:

   ld    hl, #Joy0_Down
   call  cpct_isKeyPressed_asm
   jr     z, joydown_no_pressed

         call player_down_key_actions

   joydown_no_pressed:




   ld    hl, #Key_W
   call  cpct_isKeyPressed_asm
   jr     z, w_no_pressed

         call player_up_key_actions
   
   ld    ix, #player

   w_no_pressed:

   ld    hl, #Joy0_Up
   call  cpct_isKeyPressed_asm
   jr     z, joyup_no_pressed

         call player_up_key_actions
   
   ld    ix, #player

   joyup_no_pressed:

   ld    hl, #Key_D
   call  cpct_isKeyPressed_asm
   jr     z, d_no_pressed

   call player_right_key_actions

 jr a_no_pressed

   d_no_pressed:

   ld    hl, #Joy0_Right
   call  cpct_isKeyPressed_asm
   jr     z, joyright_no_pressed

   call player_right_key_actions

 jr a_no_pressed

   joyright_no_pressed:


   ld    hl, #Joy0_Left
   ;;ld    hl, #Key_A
   call  cpct_isKeyPressed_asm
   jr     z, joyleft_no_pressed

      call player_left_key_actions
      
         jr a_no_pressed


   joyleft_no_pressed:

   ;;ld    hl, #Joy0_Left
   ld    hl, #Key_A
   call  cpct_isKeyPressed_asm
   jr     z, a_no_pressed

      call player_left_key_actions
      
         jr a_no_pressed


   a_no_pressed:

ret



;;====================================================
;;    UPDATE      -     call player_up_key_actions
;;--------------------------------------------------
;;
;; DESC.:      - Realiza las acciones de la tecla ARRIBA - JOYSTICK ARRIBA - W
;;
;; RECIBE:     - IX: Puntero a la entidad
;;
;; DESTRUYE:   - 
;;
;;====================================================
player_up_key_actions:

      call check_start_jump

ret


;;====================================================
;;    UPDATE      -     call player_up_key_actions
;;--------------------------------------------------
;;
;; DESC.:      - Realiza las acciones de la tecla ARRIBA - JOYSTICK ARRIBA - W
;;
;; RECIBE:     - IX: Puntero a la entidad
;;
;; DESTRUYE:   - 
;;
;;====================================================
player_left_key_actions:

      ld e_facing(ix), #-1 ;;El personaje ahora mira a la izquierda

      ld a, (no_move)
      cp #00
      ret nz

      ld a, (punching)
      cp #00
      ret nz

      ;;Mover al personaje:
      ld  a, (is_player_movement_allowed) ;;
      cp #0 ;;Mover al personaje estara permitido si is_player_movement_allowed vale 1. 
      ret nz


            call change_player_walking_sprite


         ld a, #-1 ;; 
         ld (player_movement), a 


   end_player_left_key_actions:
ret


;;====================================================
;;    UPDATE      -     call player_up_key_actions
;;--------------------------------------------------
;;
;; DESC.:      - Realiza las acciones de la tecla ARRIBA - JOYSTICK ARRIBA - W
;;
;; RECIBE:     - IX: Puntero a la entidad
;;
;; DESTRUYE:   - 
;;
;;====================================================
player_right_key_actions:


      ld e_facing(ix), #1 ;;El personaje ahora mira a la derecha

      ld a, (no_move)
      cp #00
      ret nz

      ld a, (punching)
      cp #00
      ret nz

      ;;Mover  al personaje:
      ld a, (is_player_movement_allowed) ;; 
      cp #0 ;;Mover al personaje estara permitido si is_screen_movement_allowed vale 0. 
      ret nz

      call change_player_walking_sprite

         ;;Si mover al personaje esta permitido:
         ld a, #1 ;; 
         ld (player_movement), a
        


ret


;;====================================================
;;    UPDATE      -     call player_up_key_actions
;;--------------------------------------------------
;;
;; DESC.:      - Realiza las acciones de la tecla ARRIBA - JOYSTICK ARRIBA - W
;;
;; RECIBE:     - IX: Puntero a la entidad
;;
;; DESTRUYE:   - 
;;
;;====================================================
player_down_key_actions:

      ld  a, e_state(ix)         ;;|
      cp  #-1                  ;;|
      ret nz

         ld a, #01
         ld (no_move), a

         ld  e_y(ix), #112
         ld  e_h(ix), #16
         ld e_state(ix), #-2

                     ;;Actualizando sprite:
                     ld a, #16 ;;Cambiamos la altura del sprite
                     ld e_h(ix), a
                     ld a, #110 ;
                     call change_player_sprite
                     ;;Fin actualizar el sprite

ret



;;====================================================
;;    UPDATE      -     call player_up_key_actions
;;--------------------------------------------------
;;
;; DESC.:      - Realiza las acciones de la tecla ARRIBA - JOYSTICK ARRIBA - W
;;
;; RECIBE:     - IX: Puntero a la entidad
;;
;; DESTRUYE:   - 
;;
;;====================================================
player_punch_key_actions:

      call check_start_hit
      ld    ix, #player

ret



;;====================================================
;;    DAMAGE PLAYER      -     call damage_player
;;--------------------------------------------------
;;
;; DESC.:      - Se actualiza la vida restando el danyo recibido.
;;
;; RECIBE:     - B: Danyo que recibe el jugador.
;;       
;;
;; DESTRUYE:   - A, B?
;;
;;====================================================
damage_player:
   ld iy, #player
   ld a, e_life(iy)
   sub b
   ld e_life(iy), a
ret




;;====================================================
;;    DEVUELVE EL NOMBRE DEL SPRITE DEL SPRITE A COLOCAR    -     call player_get_sprite_name
;;--------------------------------------------------
;;
;; DESC.:      - Si un numero es, por ejemplo, 1 te devuelve en HL el nombre del sprite que representa el 1.
;;
;;    RECIBE:     - A: Numero
;;
;;    DESTRUYE:   - HL --> Te devuelve la direccion correspondiente.
;;
;;====================================================

;;IMPORTANTE: ESTE METODO SE PUEDE OPTIMIZAR CON UNA CONCATENACION.

player_get_sprite_name:

   ;;En A estara el numero que queremos
   cp #1
   jr z, devuelve_sprite_1
   cp #0
   jr z, devuelve_sprite_0
   cp #2
   jr z, devuelve_sprite_2
   cp #3
   jr z, devuelve_sprite_3
   cp #4
   jr z, devuelve_sprite_4
   cp #5
   jr z, devuelve_sprite_5
   cp #110
   jr z, devuelve_sprite_110
   cp #111
   jr z, devuelve_sprite_111


   ;;hitbox
   cp #10
   jr z, devuelve_sprite_10
   cp #11
   jr z, devuelve_sprite_11 
   cp #112
   jr z, devuelve_sprite_112
   cp #113
   jr z, devuelve_sprite_113
   cp #114
   jr z, devuelve_sprite_114
   cp #115
   jr z, devuelve_sprite_115


   ;;Sprites de sprites1:
   devuelve_sprite_0:
   ld hl, #_sprites1_sp_00
   ret
   devuelve_sprite_1:
   ld hl, #_sprites1_sp_01
   ret
   devuelve_sprite_2:
   ld hl, #_sprites1_sp_02
   ret
   devuelve_sprite_3:
   ld hl, #_sprites1_sp_03
   ret
   devuelve_sprite_4:
   ld hl, #_sprites1_sp_04
   ret
   devuelve_sprite_5:
   ld hl, #_sprites1_sp_05
   ret
   devuelve_sprite_10:
   ld hl, #_sprites1_sp_10
   ret
   devuelve_sprite_11:
   ld hl, #_sprites1_sp_11
   ret

   ;;Sprites de sprites3:
   devuelve_sprite_110:
   ld hl, #_sprites3_sp_10
   ret
   devuelve_sprite_111:
   ld hl, #_sprites3_sp_11
   ret
   devuelve_sprite_112:
   ld hl, #_sprites3_sp_12
   ret
   devuelve_sprite_113:
   ld hl, #_sprites3_sp_13
   ret
   devuelve_sprite_114:
   ld hl, #_sprites3_sp_14
   ret
   devuelve_sprite_115:
   ld hl, #_sprites3_sp_15



ret



;;====================================================
;;    CHANGE PLAYER SPRITE      -     call damage_player
;;--------------------------------------------------
;;
;; DESC.:      - Cambia el sprite en HL
;;
;; RECIBE:     - IX: Entidad
;;             - A: Sprite que habria que dibujar en la derecha
;;       
;;
;; DESTRUYE:   - A, HL
;;
;;Basicamente este metodo evita que tengas que poner el facing en todos los metodos, automaticamente pinta un sprite en la izda o dcha.
;;====================================================

change_player_sprite:
               call player_get_sprite_name  
               ld e_sprite_image_h(ix), h
               ld e_sprite_image_l(ix), l
ret


;;====================================================
;;    CHANGE PLAYER SPRITE      -     call damage_player
;;--------------------------------------------------
;;
;; DESC.:      - Cambia el sprite en HL
;;
;; RECIBE:     - IY: Entidad
;;             - A: Sprite que habria que dibujar en la derecha
;;       
;;
;; DESTRUYE:   - A, B, HL
;;
;;Basicamente este metodo evita que tengas que poner el facing en todos los metodos, automaticamente pinta un sprite en la izda o dcha.
;;====================================================

change_hitbox_sprite:

               ld b, a
               ld a, (sword_animation)
               cp #1
               jp m, no_cambiar_sprite ;;si aun no han pasado 2 frames de animacion, no poner sin rafaga.

               ;;Cambiamos el sprite de la espada con "rafaga" al normal:
                  dec b;;lo que hace esto es cambiar al sprite "sin rafaga" de la espada, que esta justo en la posicion anterior del que lleva rafaga.

               no_cambiar_sprite:
               inc a
               ld (sword_animation),a
               ld a,b

               call player_get_sprite_name  
               ld h_sprite_image_h(iy), h
               ld h_sprite_image_l(iy), l

ret


change_player_walking_sprite:

          ;;Cambiar sprite de movimiento del personaje
            ld a, (walk_animation)
            cp #2
            jp z, player_cambiar_walking_fin

            ;;Poner sprite del segundo paso:
                  ;;Actualizando sprite:
                  ld a, #32 ;;Cambiamos la altura del sprite
                  ld e_h(ix), a
                  ld a, #1 
                  call change_player_sprite                  
                  ;;Fin actualizar el sprite   

               player_cambiar_walking_fin:
               ld a, (walk_animation)
               inc a
               ld (walk_animation), a

               cp #3
               ret nz
               ld a, #0
               ld (walk_animation), a
               ret
ret