;;-----------------------------LICENSE NOTICE------------------------------------
;;  This file is part of Crimson Knight Adventures: An Amstrad CPC Game
;;  Copyright (C) 2018 Carlos Soler Mujeriego / Enrique Vidal Cayuela / Alejandro
;;  Gutiérrez Martínez
;;
;;  This program is free software: you can redistribute it and/or modify
;;  it under the terms of the GNU Lesser General Public License as published by
;;  the Free Software Foundation, either version 3 of the License, or
;;  (at your option) any later version.
;;
;;  This program is distributed in the hope that it will be useful,
;;  but WITHOUT ANY WARRANTY; without even the implied warranty of
;;  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
;;  GNU Lesser General Public License for more details.
;;
;;  You should have received a copy of the GNU Lesser General Public License
;;  along with this program.  If not, see <http://www.gnu.org/licenses/>.
;;-------------------------------------------------------------------------------
.include "random.h.s"
.include "main.h.s"

_cpct_mxor32_seed: .dw #0x1A7A, #0x59F2 ;;Semilla. NUNCA DEBE SER 0. Sirve para generar los numeros aleatorios y deberias cambiarla.
										;; Contains the 32-bits seed for Marsaglia’s XOR-shift random number generator.
										;;  If directly accessed from ASM, do not forget to put an underscore in front (_cpct_mxor32_seed).

random_number: .db #0 		;;Contiene el ultimo numero aleatorio generado, por si queremos consultarlo.

;;====================================================
;; 	GENERATE_RANDOM_NUMBER		-		call generate_random_number
;;--------------------------------------------------
;;
;;	DESC.:		- Se genera un numero aleatorio de 8 bits en random_number y en L. 
;;				- OJO: SI LA SEMILLA ES CERO, EL PROGRAMA NO FUNCIONARÁ.
;;
;; 	RECIBE:		- Usa el valor de la semilla _cpct_mxor32_seed
;;
;; 	DESTRUYE:	- [AF, BC, DE, HL] (segun la documentacion)
;;
;;  DEVUELVE EN L EL VALOR DEL NUMERO ALEATORIO. TAMBIEN LO GUARDA EN LA VARIABLE GLOBAL random_number
;;====================================================

generate_random_number:
	call cpct_getRandom_mxor_u8_asm ;;1 de cada 4 veces, tarda casi 350 ciclos (88 microsegundos). 
									;;Las otras 3, 92 ciclos (23 microsegundos)
	ld a, l
	ld (random_number), a 			;;Actualizamos el numero aleatorio en su variable
ret

;;Si cpct_getRandom_mxor_u8_asm ralentiza mucho el programa, existen algunas alternativas mas eficientes computacionalmente pero menos aleatorias.

;;====================================================
;; 	INCREASE_RANDOM_SEED		-		call increase_random_seed
;;--------------------------------------------------
;;
;;	DESC.:		- Se genera un numero aleatorio de 8 bits en random_number y en L. 
;;				- OJO: SI LA SEMILLA ES CERO, EL PROGRAMA NO FUNCIONARÁ.
;;
;; 	RECIBE:		- Usa el valor de la semilla _cpct_mxor32_seed
;;
;; 	DESTRUYE:	- [AF, BC, DE, HL] (segun la documentacion)
;;
;;  DEVUELVE EN L EL VALOR DEL NUMERO ALEATORIO. TAMBIEN LO GUARDA EN LA VARIABLE GLOBAL random_number
;;====================================================
increase_random_seed:
	ld hl, (_cpct_mxor32_seed)
	inc hl

	ld a, h
	cp #0
	jr nz, semilla_no_es_cero
	ld a, l
	cp #0
	jr nz, semilla_no_es_cero
	;;Si la semilla es cero
	ld hl, #0001 ;;Debemos evitar que la semilla sea cero, porque cpct_getRandom_mxor_u8_asm no funcionaria.

	semilla_no_es_cero:
	ld (_cpct_mxor32_seed),hl

ret