;;-----------------------------LICENSE NOTICE------------------------------------
;;  This file is part of Crimson Knight Adventures: An Amstrad CPC Game
;;  Copyright (C) 2018 Carlos Soler Mujeriego / Enrique Vidal Cayuela / Alejandro
;;  Gutiérrez Martínez
;;
;;  This program is free software: you can redistribute it and/or modify
;;  it under the terms of the GNU Lesser General Public License as published by
;;  the Free Software Foundation, either version 3 of the License, or
;;  (at your option) any later version.
;;
;;  This program is distributed in the hope that it will be useful,
;;  but WITHOUT ANY WARRANTY; without even the implied warranty of
;;  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
;;  GNU Lesser General Public License for more details.
;;
;;  You should have received a copy of the GNU Lesser General Public License
;;  along with this program.  If not, see <http://www.gnu.org/licenses/>.
;;-------------------------------------------------------------------------------
;;============================================
;;
;;	INCLUDES
;;
;;============================================
.include "renderer.h.s"
.include "main.h.s"
.include "player.h.s"
.include "enemy.h.s"
.include "hud.h.s"

.include "mapaPruebas3_pack.h.s"
.include "mapaPruebas4_pack.h.s"
.include "mapaPruebas5_pack.h.s"

.include "HUDBackground_pack.h.s"

.area _CODE


;;====================================================
;;
;;	PRIVATE DATA
;;
;;====================================================
front_buffer: .dw #0xC000
back_buffer:  .dw #0x0000

video_ptr: .dw #0x8000

tilemap_cont: .db #0



ajuste_tilemap: .db #0 ;;Indica el numero de veces que el tilemap debe desplazarse un tile a lo largo del tiempo.
                        ;;Ejemplo, cuando me muevo al borde derecho, le pongo ajuste 3 porque quiero que a lo largo del tiempo 
                        ;;los elementos se AJUSTEN 3 veces (la primera ocurre al instante en el metodo de dibujado del tilemap).

ajuste_tilemap_lag: .db #0 ;;Retraso en iteraciones entre un ajuste y el siguiente

cortinilla_L: .db #00, #96, #04, #32, #0xFF
cortinilla_R: .db #80-4, #96, #04, #32, #0xFF


;;====================================================
;; 	RENDER ALL		-		call render_all
;;--------------------------------------------------
;;
;;	DESC.:		- Dibuja todas las entidades
;;
;; 	RECIBE:		- ...
;;
;; 	DESTRUYE:	- [HL, AF, BC, DE, IX]
;;
;;====================================================
ent_draw_all:
	ld 		ix, #player
	call    render_draw_sprite

	ld ix, #hit_hitbox
   ld a, h_exists(ix)
   cp #00
   jr z, fin_dibujar_hitbox_renderer
   ;;Dibuja la hitbox si esta existe, si a=/=0
   call render_draw_sprite

   fin_dibujar_hitbox_renderer:

	call    draw_hud
ret

ent_clear_all:
	
	ld      ix, #hud_health_bar
	ld      dw_w(ix), #10   ;; Ancho
	call	clear_box

ret


;;====================================================
;; 	DRAW HUD		-		call draw_hud
;;--------------------------------------------------
;;
;;	DESC.:		- Dibuja el HUD
;;
;; 	RECIBE:		- IX: Puntero a la entidad
;;
;; 	DESTRUYE:	- [HL, AF, BC, DE, IX]
;;
;;====================================================
draw_hud:

	ld ix, #hud_health_bar
	call	draw_box

	ld 		ix, #hud_score_1
	;;call	draw_box
   call render_draw_sprite
	ld 		ix, #hud_score_2
   call render_draw_sprite
	;;call	draw_box
	ld 		ix, #hud_score_3
   call render_draw_sprite
	;;call	draw_box
	ld 		ix, #hud_score_4
   call render_draw_sprite
	;;call	draw_box

	ld 		ix, #hud_timer_1
   call render_draw_sprite
	;;call	draw_box
	ld 		ix, #hud_timer_2
   call render_draw_sprite
	;;call	draw_box
	ld 		ix, #hud_timer_3
   call render_draw_sprite
	;;call	draw_box
	ld 		ix, #hud_timer_4
   call render_draw_sprite
	;;call	draw_box

ret

;;====================================================
;; 	DRAW BOX		-		call draw_box
;;--------------------------------------------------
;;
;;	DESC.:		- Dibuja una caja del color y forma especificado por la entidad
;;
;; 	RECIBE:		- IX: Puntero a la entidad
;;
;; 	DESTRUYE:	- [HL, AF, BC, DE, IX]
;;
;;====================================================
draw_box:
   ld    hl, (video_ptr) 
   ex    de, hl   ;; DE = Puntero a memoria
   ld     c, dw_x(ix)         ;; C = Entity X
   ld     b, dw_y(ix)         ;; B = Entity Y
   call cpct_getScreenPtr_asm
 
   ex    de, hl   ;; DE = Puntero a memoria
   ld  a, dw_color(ix)   ;; Color
   ld  b, dw_h(ix)   ;; Alto
   ld  c, dw_w(ix)   ;; Ancho
   call cpct_drawSolidBox_asm
ret

;;====================================================
;;    DRAW SPRITE    -     call draw_sprite
;;--------------------------------------------------
;;
;; DESC.:      - Dibuja un sprite con el tamanyo especificado por la entidad
;;
;;    RECIBE:     - IX: Puntero a la entidad
;;
;;    DESTRUYE:   - [HL, AF, BC, DE, IX] 
;;
;;====================================================

draw_enemy_sprite:

   ;; Check if enemy X position is between 0 and 80
   ld a, en_x(ix)       ;; A = PX
   cp #0
   jp m, end_draw_enemy
   ld b, a              ;; B = PX
   ld a, en_w(ix)       ;; A = PW
   add a, b             ;; PX + PW
   ld b, a              ;; B = PX + PW
   ld a, #79
   sub b                ;; 79 - (PX + PW)
   jp m, end_draw_enemy
   
   call render_draw_sprite

   end_draw_enemy:
   
ret

;;====================================================
;; 	CLEAR BOX		-		call clear_box
;;--------------------------------------------------
;;
;;	DESC.:		- Dibuja una caja del color y forma especificado por la entidad
;;
;; 	RECIBE:		- IX: Puntero a la entidad
;;
;; 	DESTRUYE:	- [HL, AF, BC, DE, IX]
;;
;;====================================================
clear_box:
	ld  a, dw_color(ix)
	ex af, af'

	ld  dw_color(ix), #0xFF

	call draw_box
	ex af, af'
	ld dw_color(ix), a
ret


render_draw_sprite:
	
   ld    hl, (video_ptr) 
   ex    de, hl   ;; DE = Puntero a memoria
   ld     c, dw_x(ix)         ;; C = Entity X
   ld     b, dw_y(ix)         ;; B = Entity Y
   call cpct_getScreenPtr_asm
 
   ex    de, hl   ;; DE = Puntero a memoria
   ld h, dw_sp_h(ix)
   ld l, dw_sp_l(ix)
   ld  b, dw_h(ix)   ;; Alto
   ld  c, dw_w(ix)   ;; Ancho
   call cpct_drawSprite_asm
ret


;;====================================
;; FUNCION QUE PINTA UN STRING
;;    ENTRADA: IX --> entidad string 
;;    DESTRUYE: ALL REGISTERS
;;=====================================
render_draw_string:
   ;; Set up draw char colours before calling draw string
   ld    h, st_bp(ix)       ;; H = Background PEN (0)
   ld    l, st_fp(ix)       ;; L = Foreground PEN (3)

   call cpct_setDrawCharM0_asm   ;; Set draw char colours

   ;; Calculate a video-memory location for printing a string
   ld    hl, (video_ptr)
   ex    de, hl         ;; DE = Pointer to start of the screen
   ld    b, st_y(ix)                  ;; B = y coordinate (24 = 0x18)
   ld    c, st_x(ix)               ;; C = x coordinate (16 = 0x10)

   call cpct_getScreenPtr_asm    ;; Calculate video memory location and return it in HL
   
   ld    a, st_h(ix)
   ld    b, a
   ld    a, st_l(ix)
   ld    c, a
   ld    iy, #0x0000
   add   iy, bc

   ;; Print the string in video memory
   ;; HL already points to video memory, as it is the return
   ;; value from cpct_getScreenPtr_asm
   ;;ld   iy, #string      ;; IY = Pointer to the string 
   call cpct_drawStringM0_asm    ;; Draw the string
ret

;;================================================================================
;; DRAW A CHARACTER
;; 
;; (2B HL) video_memory Video memory location where the character will be drawn
;; (1B E ) ascii  Character to be drawn (ASCII code)
;;    
;;    DECIMAL        ASCII
;;    --------------------
;;       0           48
;;       1           49
;;     ...          ...
;;       9           57
;;
;;    INPUT VALUES:  A -> NUMBER IN DECIMAL 
;;                   B -> COORDINATE Y
;;                   C -> COORDINATE X                   
;;================================================================================
draw_number:
   ld (number), a
   ld a, b
   ld (coordinate_y), a
   ld a, c
   ld (coordinate_x), a

   ;; Set up draw char colours before calling draw string
   ld    h, #15      ;; H = Background PEN (0)
   ld    l, #4       ;; L = Foreground PEN (3)
   call cpct_setDrawCharM0_asm   ;; Set draw char colours

   ;; Calculate a video-memory location for printing a string
   ld    hl, (video_ptr)
   ex    de, hl                     ;; DE = Pointer to start of the screen
   coordinate_y = . + 1
   ld    b, #0x00                   ;; B = y coordinate (24 = 0x18)
   coordinate_x = . + 1
   ld    c, #0x00                   ;; C = x coordinate (16 = 0x10)
   call cpct_getScreenPtr_asm       ;; Calculate video memory location and return it in HL

   ;; CONVERT NUMBER IN
   number = . + 1
   ld a, #0
   ld b, #48
   add a, b
   ld e, a
   call cpct_drawCharM0_asm

ret

;;====================================================
;; 	INITIALIZE TILEMAP		-		call initialize_tilemap
;;--------------------------------------------------
;;
;;	DESC.:		- Inicializamos el viewport del mapa que vamos a dibujar
;;
;; 	RECIBE:		- Nada.
;;
;; 	DESTRUYE:	- [HL, AF, BC, DE]
;;
;;====================================================

initialize_tilemap:
   	;;B: Alto en tiles DE LA VIEWPORT, DE LO QUE QUEREMOS DIBUJAR.
   	;;C: Ancho en tiles DE LA VIEWPORT, DE LO QUE QUEREMOS DIBUJAR.
   	;;DE: Tile_Map_Width, definido en el archivo de nivel .h

   ld a, (actual_level)

   cp #7
   jr z, initialize_map_5
   cp #6
   jr z, initialize_map_4
   cp #5
   jr z, initialize_map_4
   cp #4
   jr z, initialize_map_3
   cp #3
   jr z, initialize_map_2
   cp #2
   jr z, initialize_map_1
   cp #1
   jr z, initialize_map_1

      initialize_map_5:

      ld de, #_mapaPrueba5_W      ;;Tilemap width ULTRAIMPORTANTE: HAY QUE PONER MANUALMENTE == en vez de = EN EL ARCHIVO .H.S DEL NIVEL.
      ld hl, #tileset_ptr_5        ;;Puntero al tileset, calculado arriba.

      jr initialize_tilemap_end

      initialize_map_4:

      ld de, #_mapaPrueba4_W      ;;Tilemap width ULTRAIMPORTANTE: HAY QUE PONER MANUALMENTE == en vez de = EN EL ARCHIVO .H.S DEL NIVEL.
      ld hl, #tileset_ptr_4        ;;Puntero al tileset, calculado arriba.

      jr initialize_tilemap_end

      initialize_map_3:

      ld de, #_mapaPrueba3_W      ;;Tilemap width ULTRAIMPORTANTE: HAY QUE PONER MANUALMENTE == en vez de = EN EL ARCHIVO .H.S DEL NIVEL.
      ld hl, #tileset_ptr_3        ;;Puntero al tileset, calculado arriba.

      jr initialize_tilemap_end


      initialize_map_2:


      ld de, #_mapaPrueba2_W      ;;Tilemap width ULTRAIMPORTANTE: HAY QUE PONER MANUALMENTE == en vez de = EN EL ARCHIVO .H.S DEL NIVEL.
      ld hl, #tileset_ptr_2        ;;Puntero al tileset, calculado arriba.

      jr initialize_tilemap_end

      initialize_map_1:

      ld de, #_mapaPrueba_W      ;;Tilemap width ULTRAIMPORTANTE: HAY QUE PONER MANUALMENTE == en vez de = EN EL ARCHIVO .H.S DEL NIVEL.
      ld hl, #tileset_ptr        ;;Puntero al tileset, calculado arriba.


      initialize_tilemap_end:
      ld bc, #0x1012             ;;Viewport de 10x18 TILES.
      call cpct_etm_setDrawTilemap4x8_ag_asm    ;;Configuramos la funcion de dibujado para dibujar un tilemap.


ret

;;====================================================
;; 	DRAW TILEMAP		-		call draw_tilemap
;;--------------------------------------------------
;;
;;	DESC.:		- Dibuja el tilemap
;;
;; 	RECIBE:		- Nada.
;;
;; 	DESTRUYE:	- [HL, AF, BC, DE, IX, IY]?¿ Destruye menos registros que esos.
;;
;;====================================================
draw_tilemap:
	;;**Pintado de tilemap**:
    ;;ld hl, #0xC004 

   ld hl, (video_ptr)
   ld  l, #0xA4
   ;;ld hl, #0xC0A4                         ;;En modo 0, la pantalla ocupa 80/4 = 20 tiles. Cada tile ocupa 4 bytes.
                                          ;;Si tu tilemap no ocupa "toda la pantalla", tenlo en cuenta para centrarlo.
                                          ;;IMPORTANTE: HL debe ser multiplo de 4 y debe ser fila 0 de la fila de caracteres o el programa crasheara.
   
   ld a, (actual_level)
   cp #2

         ld a, (player_screen_position)
         cp #10

         jr z, tilemap_move_left
         cp #-10
         jr z, tilemap_move_right

         jr end_tilemap_loop

            ;;TILEMAP SE MUEVE A LA IZQUIERDA
            tilemap_move_right: 


            ld a, (screen_scroll_position)  ;; ESTO DEBES REACTIVARLO LUEGO PARA QUE NO PUEDAS MOVERTE A LA IZQUIERDA
             cp #00
             jr  z, end_tilemap_loop 

            ld a, #4
            ld (ajuste_tilemap), a

            ld a, #0
            ld (ajuste_tilemap_lag), a

            jr end_tilemap_loop
            ;;--------------------

            ;;TILEMP SE MUEVE A LA DERECHA
            tilemap_move_left: 


            ld a, (end_level_position)
            ld b,a 

            ld a, (screen_scroll_position)

            cp b ;;ESTA LINEA DE CODIGO DEBE SER EDITADA ESTA LINEA DE CODIGO DEBE SER EDITADA ESTA LINEA DE CODIGO DEBE SER EDITADA
            jr  z, end_tilemap_loop 
            
            ld a, #-4
            ld (ajuste_tilemap), a

            ld a, #0
            ld (ajuste_tilemap_lag), a

            jr end_tilemap_loop

   end_tilemap_loop:

      ;;//**Fin de Pintado de tilemap**.


      ld a, (ajuste_tilemap_lag)
      cp #0
      jr nz, no_ejecutar_adjust_tilemap

         call adjust_tilemap

         ld a, #1
         ld (ajuste_tilemap_lag), a

   no_ejecutar_adjust_tilemap:

      ld a, (ajuste_tilemap_lag)
      dec a 
      ld (ajuste_tilemap_lag), a


      ld hl, (video_ptr)
      ld  l, #0xA4
      ld de, (tilemap)     ;;Puntero al inicio del tilemap. El inicio marca desde donde se empieza a dibujar. Si lo cambias, puedes "desplazarte" por el mapa.
      call cpct_etm_drawTilemap4x8_ag_asm ;;Pintado del tilemap



ret

;;====================================================
;;    ADJUST TILEMAP      -     call adjust_tilemap
;;--------------------------------------------------
;;
;; DESC.:      - Cada X iteraciones (controladas en main), las entidades se ajustarán. 
;;               El numero de veces que se deben ajustar esta definido en ajuste_tilemap.
;;
;;                Si ajuste tilemap es negativo, las entidades se ajustan a la IZQUIERDA (porque la camara se movio a la derecha)
;;                Si ajuste tilemap es positivo, las entidades se ajustan a la DERECHA (porque la camara se movio a la izquierda)
;;
;;    RECIBE:     - 
;;
;;    DESTRUYE:   - [A]
;;
;;====================================================
adjust_tilemap:

   tilemap = . + 1         ;;Nos permitira mover la pantalla.             
   ld de, #tilemap_ptr     ;;Puntero al inicio del tilemap. El inicio marca desde donde se empieza a dibujar. Si lo cambias, puedes "desplazarte" por el mapa.



   ld a, (ajuste_tilemap)      
   cp #0
   ret z
   jp m, ajuste_entidades_a_la_izquierda


   dec a
   ld (ajuste_tilemap), a

   ld a, (screen_scroll_position)
   dec a
   ld (screen_scroll_position), a

   ld hl, (tilemap)
   dec hl ;;***Importante: esto habria que cambiarlo si queremos que el tilemap se mueva mas rapido
   ld (tilemap), hl   

   ;;Ajuste entidades a la derecha
   ld a, #4

   jr ajuste_entidades_fin

   ajuste_entidades_a_la_izquierda:

      inc a 
      ld (ajuste_tilemap),a

      ld a, (screen_scroll_position) ;;Yendo hacia la derecha, screen_scroll aumenta.
      inc a
      ld (screen_scroll_position), a

      ld hl, (tilemap)
      inc hl ;;***Importante: esto habria que cambiarlo si queremos que el tilemap se mueva mas rapido
      ld (tilemap), hl

      ld a, #-4


   ajuste_entidades_fin:

   ;;Realizamos el ajuste
      ld d, a ;;D lo necesitan los enemigos para ajustarse bien
      call tilemap_update_entities_positions


ret

;;====================================================
;;    RESTART TILEMAP      -     call restart_tilemap
;;--------------------------------------------------
;;
;; DESC.:      - Reinicia el tilemap (pone el tilemap en la posicion incial)
;;
;;    RECIBE:     - Nada.
;;
;;    DESTRUYE:   - [HL], [A]
;;
;;====================================================

tilemap_update_entities_positions:

   ;;Estos metodos reciben en A el numero de pixeles positivos o negativos a desplazarse:

   ;;Update player
   call player_move_along_tilemap
   ;;Update enemies
   call enemies_move_along_tilemap

ret

;;====================================================
;;    RESTART TILEMAP      -     call restart_tilemap
;;--------------------------------------------------
;;
;; DESC.:      - Reinicia el tilemap (pone el tilemap en la posicion incial)
;;
;;    RECIBE:     - Nada.
;;
;;    DESTRUYE:   - [HL], [A]
;;
;;====================================================

;;IMPORTANTE: Esto habra que desglosarlo por niveles mas adelante
restart_tilemap:

   ld hl, #tilemap_ptr         ;;Reiniciamos el tilemap con el valor inicial del tilemap
   ld (tilemap), hl

   ld a, #0
   ld (is_player_movement_allowed), a  ;;Permitimos al jugador moverse de nuevo.
   ld a, #0
   ld (screen_scroll_position), a      ;;Reiniciamos la posicion relativa del jugador al mapa.
ret

;;====================================================
;;    RESTART TILEMAP      -     call restart_tilemap
;;--------------------------------------------------
;;
;; DESC.:      - Reinicia el tilemap (pone el tilemap en la posicion incial)
;;
;;    RECIBE:     - A: Nivel que queremos cargar
;;
;;    DESTRUYE:   - [HL], [A]
;;
;;====================================================

;;IMPORTANTE: Esto habra que desglosarlo por niveles mas adelante
change_tilemap:

      cp #7
      jr z, cargar_nivel_7
      cp #6
      jr z, cargar_nivel_5_6
      cp #5
      jr z, cargar_nivel_5_6
      cp #4
      jr z, cargar_nivel_4
   	cp #3
   	jr z, cargar_nivel_3
      cp #2
      jr z, cargar_nivel_1_2
      cp #1
   	jr z, cargar_nivel_1_2

      cargar_nivel_7:

         ;;Descompresion de los mapas y los tilesets
         
         ld hl, #_mapaPruebas5_pack_end ;;definido en mapaPruebas_pack.s ;;DECOMPRESS-TILE TESTS
         ;;ld hl, #399
         ld de, #decompress_buffer_end5 ;;Define donde acaba el bucle de descompresion y donde empieza el codigo. ;;DECOMPRESS-TILE TESTS
         call cpct_zx7b_decrunch_s_asm ;;DECOMPRESS-TILE TESTS

         call initialize_tilemap
        
         ld a, (end_level_scroll_map_7)
         ld (end_level_position), a

         jr cargar_nivel_fin

      cargar_nivel_5_6:

         ;;Descompresion de los mapas y los tilesets
         
         ld hl, #_mapaPruebas4_pack_end ;;definido en mapaPruebas_pack.s ;;DECOMPRESS-TILE TESTS
         ;;ld hl, #399
         ld de, #decompress_buffer_end4 ;;Define donde acaba el bucle de descompresion y donde empieza el codigo. ;;DECOMPRESS-TILE TESTS
         call cpct_zx7b_decrunch_s_asm ;;DECOMPRESS-TILE TESTS

         call initialize_tilemap
        
         ld a, (end_level_scroll_map_5_6)
         ld (end_level_position), a

         jr cargar_nivel_fin


      cargar_nivel_4:

         ;;Descompresion de los mapas y los tilesets
         
         ld hl, #_mapaPruebas3_pack_end ;;definido en mapaPruebas_pack.s ;;DECOMPRESS-TILE TESTS
         ;;ld hl, #399
         ld de, #decompress_buffer_end3 ;;Define donde acaba el bucle de descompresion y donde empieza el codigo. ;;DECOMPRESS-TILE TESTS
         call cpct_zx7b_decrunch_s_asm ;;DECOMPRESS-TILE TESTS

         call initialize_tilemap
        
         ld a, (end_level_scroll_map_4)
         ld (end_level_position), a

         jr cargar_nivel_fin


      cargar_nivel_3:


         ;;Descompresion de los mapas y los tilesets
         
         ld hl, #_mapaPruebas2_pack_end ;;definido en mapaPruebas_pack.s ;;DECOMPRESS-TILE TESTS
         ;;ld hl, #399
         ld de, #decompress_buffer_end2 ;;Define donde acaba el bucle de descompresion y donde empieza el codigo. ;;DECOMPRESS-TILE TESTS
         call cpct_zx7b_decrunch_s_asm ;;DECOMPRESS-TILE TESTS

         call initialize_tilemap
        
         ld a, (end_level_scroll_map_3)
         ld (end_level_position), a

         jr cargar_nivel_fin

      cargar_nivel_1_2:

         ;;Descompresion de los mapas y los tilesets
         ld hl, #_mapaPruebas_pack_end ;;definido en mapaPruebas_pack.s ;;DECOMPRESS-TILE TESTS
         ld de, #decompress_buffer_end ;;Define donde acaba el bucle de descompresion y donde empieza el codigo. ;;DECOMPRESS-TILE TESTS
         call cpct_zx7b_decrunch_s_asm ;;DECOMPRESS-TILE TESTS

         call initialize_tilemap
        
         ld a, (end_level_scroll_map_1_2)
         ld (end_level_position), a


   cargar_nivel_fin: ;;cosas que pasan cuando has cargado el nivel:

   call restart_tilemap    ;;tenemos que reiniciar varios parametros

ret

clear_buffer_8:

	ld hl, #0x8000
   	ld de, #0x8001
   	ld (hl), #0xff
   	ld bc, #0x4000
   	ldir

ret

clear_buffer_C:

	ld hl, #0xC000
   	ld de, #0xC001
   	ld (hl), #0xff
   	ld bc, #0x4000
   	ldir

ret

switch_buffers:

	modifier = .+1
	ld   l, #0x20
	call cpct_setVideoMemoryPage_asm
	ld  hl, #modifier
	ld   a, #0x10
	xor (hl)
	ld  (modifier), a

	ld  hl, #video_ptr+1
	ld   a, #0x40
	xor (hl)
	ld  (video_ptr+1), a

ret

draw_HUDBackground:

	ld hl, #_HUDBackground_pack_end ;;definido en mapaPruebas_pack.s ;;DECOMPRESS-TILE TESTS
   	ld de, #0xC000-1 ;;Define donde acaba el bucle de descompresion y donde empieza el codigo. ;;DECOMPRESS-TILE TESTS
   	call cpct_zx7b_decrunch_s_asm ;;DECOMPRESS-TILE TESTS

   	ld hl, #_HUDBackground_pack_end ;;definido en mapaPruebas_pack.s ;;DECOMPRESS-TILE TESTS
   	ld de, #0xFFFF ;;Define donde acaba el bucle de descompresion y donde empieza el codigo. ;;DECOMPRESS-TILE TESTS
   	call cpct_zx7b_decrunch_s_asm ;;DECOMPRESS-TILE TESTS



   ld       ix, #hud_helmet_1
   ld a, #1
   call update_helmet_icon
   call render_draw_sprite
   
   ld       ix, #hud_helmet_2
   ld a, #2
   call update_helmet_icon
   call render_draw_sprite
   
   ld       ix, #hud_helmet_3
   ld a, #3
   call update_helmet_icon
   call render_draw_sprite

   call switch_buffers
      
   ld       ix, #hud_helmet_1
   call render_draw_sprite
   ld       ix, #hud_helmet_2
   call render_draw_sprite
   ld       ix, #hud_helmet_3
   call render_draw_sprite

   call switch_buffers


ret

draw_cortinilla:

   ld      ix, #cortinilla_R
   call  draw_box

   ld      ix, #cortinilla_L
   call  draw_box

ret
