;;-----------------------------LICENSE NOTICE------------------------------------
;;  This file is part of Crimson Knight Adventures: An Amstrad CPC Game
;;  Copyright (C) 2018 Carlos Soler Mujeriego / Enrique Vidal Cayuela / Alejandro
;;  Gutiérrez Martínez
;;
;;  This program is free software: you can redistribute it and/or modify
;;  it under the terms of the GNU Lesser General Public License as published by
;;  the Free Software Foundation, either version 3 of the License, or
;;  (at your option) any later version.
;;
;;  This program is distributed in the hope that it will be useful,
;;  but WITHOUT ANY WARRANTY; without even the implied warranty of
;;  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
;;  GNU Lesser General Public License for more details.
;;
;;  You should have received a copy of the GNU Lesser General Public License
;;  along with this program.  If not, see <http://www.gnu.org/licenses/>.
;;-------------------------------------------------------------------------------
.include "main.h.s"
.include "renderer.h.s"
.include "screens.h.s"
.include "random.h.s"
.include "time.h.s"
.include "hud.h.s"
.include "titleScreen_pack.h.s"
.include "gameOver_pack.h.s"
.include "youWin_pack.h.s"
.include "credits_pack.h.s"

.area CODE
;;	PRESS KEY STRING & VARS
string: .asciz "Press Any Key"
DefineString press_string, #14, #130, #15, #4, string
color_1: .db #8
color_0: .db #4
color_state: .db #0 ;; 0 = Color 0, 1 = Color 1
color_cont:  .db #0

;;
;; 	SCREEN COMMON ROUTINES
;;

;;===================================
;; 	LOOPS UNTIL ANY KEY IS PRESSED
;;	AND DOES THE ROUTINE INSIDE
;; 	DESTROY:
;; 	ENTRY: HL --> POINTER TO ROUTINE
;;===================================
screen_loop:
	ld (method), hl

	ld a, #0
   	ld (music_play), a

	;; Wait ~ 1s
	ld b, #2
	ld c, #100
	call wait_time

	method = . + 1
   	call #0x0000

   	call cpct_waitVSYNC_asm         ;; Wait raster
    call switch_buffers        		;; Wait raster


   	loop:
   	;;----------------------------
   	call press_string_change_color 		;; Cambia el color cada X tiempo
   	ld 	ix, #press_string
	call render_draw_string
   	;;----------------------------
   	call cpct_waitVSYNC_asm         ;; Wait raster
    call switch_buffers        		;; Wait raster
    call cpct_isAnyKeyPressed_asm 	;; Busca si alguna tecla esta pulsada
	cp #0
	jr z, loop
	call clear_buffer_8
   	call clear_buffer_C

   	ld a, #1
   	ld (music_play), a
ret

;;===================================
;; 	TITLE SCREEN
;; 	DESTROY:
;; 	ENTRY:		HL -> POINTER TO THE IMAGE
;;===================================

show_title_screen:
	ld (screen), hl
	;;ld hl, #_titleScreen_pack_end ;;definido en mapaPruebas_pack.s ;;DECOMPRESS-TILE TESTS
   	ld de, #0xC000-1 ;;Define donde acaba el bucle de descompresion y donde empieza el codigo. ;;DECOMPRESS-TILE TESTS
   	call cpct_zx7b_decrunch_s_asm ;;DECOMPRESS-TILE TESTS
   	screen = . + 1
   	ld hl, #0x0000
   	;;ld hl, #_titleScreen_pack_end ;;definido en mapaPruebas_pack.s ;;DECOMPRESS-TILE TESTS
   	ld de, #0xFFFF ;;Define donde acaba el bucle de descompresion y donde empieza el codigo. ;;DECOMPRESS-TILE TESTS
   	call cpct_zx7b_decrunch_s_asm ;;DECOMPRESS-TILE TESTS

ret




;;===================================
;; 	CHANGE THE COLOR OF THE PRESS STRING
;; 	DESTROY:
;; 	ENTRY:
;;===================================
press_string_change_color:
ld a, (color_cont)
cp #0
jp nz, end_change

	ld ix, #press_string

	ld a, (color_state)
	cp #0
	jp z, goto_color_1 ;; Si el estado es 0, color inicial cambia

	ld a, (color_0)
	ld st_fp(ix), a
	ld a, #0
	ld (color_state), a

	jp end

	goto_color_1:

	ld a, (color_1)
	ld st_fp(ix), a
	ld a, #1
	ld (color_state), a

	end:
	ld a, #0x0A
	end_change:
	dec a
	ld (color_cont), a
ret

;;
;;	TITLE SCREEN ROUTINES
;;

;; title screen sprite
;;EntityDrawable title_screen_sprite, #24, #50 , #32 , #64 , #0000

title_screen_update:
   	call increase_random_seed 			;; Cambiamos la semilla del numero aleatorio a generar.
   	;;ld 	ix, #title_screen_sprite
	;;call render_draw_sprite
	
ret

title_screen_loop:
	;; LOOP TO WAIT
	ld hl, #_titleScreen_pack_end 
	call show_title_screen
	ld hl, #title_screen_update
	call screen_loop
	
ret

;;
;; 	WIN SCREEN ROUTINES
;;
win_screen_update:
ret


win_screen_loop:
	
	ld hl, #_youWin_pack_end 
	call show_title_screen
	ld hl, #win_screen_update
	call screen_loop

	call credits_loop
ret


;;
;;	GAME OVER SCREEN
;;
game_over_screen_update:
ret

game_over_screen_loop:

	call hud_draw_all_score
	
	ld hl, #_gameOver_pack_end
	call show_title_screen

	ld hl, #game_over_screen_update
	call screen_loop
ret


;;
;; 	NEXT LEVEL ROUTINES
;;

next_level_string: .asciz "Level Complete"
DefineString next_level, #12, #50, #15, #10, next_level_string

next_level_screen_update:
	;; DIBULAMOS LA WIN SCREEN
	ld ix, #next_level
	call render_draw_string

	;;call hud_draw_all_score
	call hud_update_all_score
ret

next_level_screen_loop:

	;; call next_level_screen_update

	;; DIBULAMOS LA WIN SCREEN
	ld ix, #next_level
	call render_draw_string

	call hud_draw_all_score
	call cpct_waitVSYNC_asm         ;; Wait raster
    call switch_buffers 

	ld hl, #next_level_screen_update
	call screen_loop
ret


credits_loop:

	ld hl, #_credits_pack_end
	call show_title_screen

	call cpct_waitVSYNC_asm         ;; Wait raster
    call switch_buffers        		;; Wait raster

   	ld hl, #_credits_pack_end
	call show_title_screen

	call cpct_waitVSYNC_asm         ;; Wait raster
    call switch_buffers        		;; Wait raster

	;; Wait ~ 1s
	ld b, #30
	ld c, #100
	call wait_time

ret